@@ -12,10 +12,14 @@ from dagster.cli.workspace.cli_target import (
     get_repository_location_from_kwargs,
     repository_target_argument,
 )
-from dagster.core.definitions.run_request import JobType
+from dagster.core.definitions.run_request import InstigatorType
 from dagster.core.host_representation import ExternalRepository
 from dagster.core.instance import DagsterInstance
-from dagster.core.scheduler.job import JobState, JobStatus, SensorJobData
+from dagster.core.scheduler.instigation import (
+    InstigatorState,
+    InstigatorStatus,
+    SensorInstigatorData,
+)
 from dagster.utils.error import serializable_error_info_from_exc_info
 
 
@@ -28,7 +32,7 @@ def sensor_cli():
 
 def print_changes(external_repository, instance, print_fn=print, preview=False):
     sensor_states = instance.all_stored_job_state(
-        external_repository.get_origin_id(), JobType.SENSOR
+        external_repository.get_origin_id(), InstigatorType.SENSOR
     )
     external_sensors = external_repository.get_external_sensors()
     external_sensors_dict = {s.get_external_origin_id(): s for s in external_sensors}
@@ -138,47 +142,42 @@ def execute_list_command(running_filter, stopped_filter, name_filter, cli_args,
                 print_fn(title)
                 print_fn("*" * len(title))
 
-            stored_sensors_by_id = {}
-            for job_state in instance.all_stored_job_state(
-                external_repo.get_external_origin_id(), JobType.SENSOR
-            ):
-                stored_sensors_by_id[job_state.job_origin_id] = job_state
-
-            all_state = [
-                stored_sensors_by_id.get(
-                    external_sensor.get_external_origin_id(),
-                    external_sensor.get_default_job_state(instance),
+            repo_sensors = external_repo.get_external_sensors()
+            stored_sensors_by_origin_id = {
+                stored_sensor_state.job_origin_id: stored_sensor_state
+                for stored_sensor_state in instance.all_stored_job_state(
+                    external_repo.get_external_origin_id(), job_type=InstigatorType.SENSOR
                 )
-                for external_sensor in external_repo.get_external_sensors()
-            ]
-
-            if running_filter:
-                jobs = [
-                    job_state for job_state in all_state if job_state.status == JobStatus.RUNNING
-                ]
-            elif stopped_filter:
-                jobs = [
-                    job_state for job_state in all_state if job_state.status == JobStatus.STOPPED
-                ]
-            else:
-                jobs = all_state
+            }
 
             first = True
 
-            for job_state in jobs:
-                # If --name filter is present, only print the job name
-                if name_filter:
-                    print_fn(job_state.job_name)
+            for external_sensor in repo_sensors:
+                stored_sensor_state = stored_sensors_by_origin_id.get(
+                    external_sensor.get_external_origin_id()
+                )
+                if running_filter and (
+                    not stored_sensor_state
+                    or stored_sensor_state.status == InstigatorStatus.STOPPED
+                ):
+                    continue
+                if stopped_filter and stored_sensor_state and InstigatorStatus.RUNNING:
                     continue
 
-                flag = "[{status}]".format(status=job_state.status.value) if job_state else ""
-                job_title = "Sensor: {name} {flag}".format(name=job_state.job_name, flag=flag)
+                if name_filter:
+                    print_fn(external_sensor.name)
+                    continue
 
+                status = (
+                    stored_sensor_state.status if stored_sensor_state else InstigatorStatus.STOPPED
+                )
+                sensor_title = f"Sensor: {external_sensor.name} [{status.value}]"
                 if not first:
-                    print_fn("*" * len(job_title))
+                    print_fn("*" * len(sensor_title))
+
                 first = False
 
-                print_fn(job_title)
+                print_fn(sensor_title)
 
 
 @sensor_cli.command(name="start", help="Start an existing sensor.")
@@ -292,7 +291,7 @@ def execute_preview_command(
                         last_run_key,
                         cursor,
                     )
-                except Exception:  # pylint: disable=broad-except
+                except Exception:
                     error_info = serializable_error_info_from_exc_info(sys.exc_info())
                     print_fn(
                         "Failed to resolve sensor for {sensor_name} : {error_info}".format(
@@ -366,11 +365,11 @@ def execute_cursor_command(sensor_name, cli_args, print_fn):
             job_state = instance.get_job_state(external_sensor.get_external_origin_id())
             if not job_state:
                 instance.add_job_state(
-                    JobState(
+                    InstigatorState(
                         external_sensor.get_external_origin(),
-                        JobType.SENSOR,
-                        JobStatus.STOPPED,
-                        SensorJobData(
+                        InstigatorType.SENSOR,
+                        InstigatorStatus.STOPPED,
+                        SensorInstigatorData(
                             min_interval=external_sensor.min_interval_seconds, cursor=cursor_value
                         ),
                     )
@@ -378,7 +377,7 @@ def execute_cursor_command(sensor_name, cli_args, print_fn):
             else:
                 instance.update_job_state(
                     job_state.with_data(
-                        SensorJobData(
+                        SensorInstigatorData(
                             last_tick_timestamp=job_state.job_specific_data.last_tick_timestamp,
                             last_run_key=job_state.job_specific_data.last_run_key,
                             min_interval=external_sensor.min_interval_seconds,
