@@ -1,25 +1,28 @@
 # encoding: utf-8
 import hashlib
-from typing import Any, Dict
+from typing import TYPE_CHECKING, Any, Dict, List
 
 from dagster import check
 from dagster.core.errors import DagsterInvalidConfigDefinitionError
 
-from .config_type import ConfigType, ConfigTypeKind
+from .config_type import Array, ConfigType, ConfigTypeKind
+
+if TYPE_CHECKING:
+    from dagster.config.field import Field
 
 
-def all_optional_type(config_type):
+def all_optional_type(config_type: ConfigType) -> bool:
     check.inst_param(config_type, "config_type", ConfigType)
 
     if ConfigTypeKind.is_shape(config_type.kind):
-        for field in config_type.fields.values():
+        for field in config_type.fields.values():  # type: ignore
             if field.is_required:
                 return False
         return True
 
     if ConfigTypeKind.is_selector(config_type.kind):
-        if len(config_type.fields) == 1:
-            for field in config_type.fields.values():
+        if len(config_type.fields) == 1:  # type: ignore
+            for field in config_type.fields.values():  # type: ignore
                 if field.is_required:
                     return False
             return True
@@ -63,7 +66,7 @@ def _add_hash(m, string):
     m.update(string.encode("utf-8"))
 
 
-def _compute_fields_hash(fields, description):
+def _compute_fields_hash(fields, description, field_aliases=None):
 
     m = hashlib.sha1()  # so that hexdigest is 40, not 64 bytes
     if description:
@@ -79,11 +82,19 @@ def _compute_fields_hash(fields, description):
         if field.description:
             _add_hash(m, ":description: " + field.description)
 
+    field_aliases = check.opt_dict_param(
+        field_aliases, "field_aliases", key_type=str, value_type=str
+    )
+    for field_name in sorted(list(field_aliases.keys())):
+        field_alias = field_aliases[field_name]
+        _add_hash(m, ":fieldname: " + field_name)
+        _add_hash(m, ":fieldalias: " + field_alias)
+
     return m.hexdigest()
 
 
-def _define_shape_key_hash(fields, description):
-    return "Shape." + _compute_fields_hash(fields, description)
+def _define_shape_key_hash(fields, description, field_aliases):
+    return "Shape." + _compute_fields_hash(fields, description, field_aliases=field_aliases)
 
 
 class Shape(_ConfigHasFields):
@@ -95,27 +106,40 @@ class Shape(_ConfigHasFields):
     Args:
         fields (Dict[str, Field]):
             The specification of the config dict.
+        field_aliases (Dict[str, str]):
+            Maps a string key to an alias that can be used instead of the original key. For example,
+            an entry {"solids": "ops"} means that someone could use "ops" instead of "solids" as a
+            top level string key.
     """
 
     def __new__(
         cls,
         fields,
         description=None,
+        field_aliases=None,
     ):
         return _memoize_inst_in_field_cache(
             cls,
             Shape,
-            _define_shape_key_hash(expand_fields_dict(fields), description),
+            _define_shape_key_hash(expand_fields_dict(fields), description, field_aliases),
         )
 
-    def __init__(self, fields, description=None):
+    def __init__(
+        self,
+        fields,
+        description=None,
+        field_aliases=None,
+    ):
         fields = expand_fields_dict(fields)
         super(Shape, self).__init__(
             kind=ConfigTypeKind.STRICT_SHAPE,
-            key=_define_shape_key_hash(fields, description),
+            key=_define_shape_key_hash(fields, description, field_aliases),
             description=description,
             fields=fields,
         )
+        self.field_aliases = check.opt_dict_param(
+            field_aliases, "field_aliases", key_type=str, value_type=str
+        )
 
 
 def _define_permissive_dict_key(fields, description):
@@ -140,9 +164,9 @@ class Permissive(_ConfigHasFields):
 
     .. code-block:: python
 
-        @solid(config_schema=Field(Permissive({'required': Field(String)})))
+        @op(config_schema=Field(Permissive({'required': Field(String)})))
         def partially_specified_config(context) -> List:
-            return sorted(list(context.solid_config.items()))
+            return sorted(list(context.op_config.items()))
     """
 
     def __new__(cls, fields=None, description=None):
@@ -187,7 +211,7 @@ class Selector(_ConfigHasFields):
 
     .. code-block:: python
 
-        @solid(
+        @op(
             config_schema=Field(
                 Selector(
                     {
@@ -201,12 +225,12 @@ class Selector(_ConfigHasFields):
             )
         )
         def hello_world_with_default(context):
-            if 'haw' in context.solid_config:
-                return 'Aloha {whom}!'.format(whom=context.solid_config['haw']['whom'])
-            if 'cn' in context.solid_config:
-                return '你好，{whom}!'.format(whom=context.solid_config['cn']['whom'])
-            if 'en' in context.solid_config:
-                return 'Hello, {whom}!'.format(whom=context.solid_config['en']['whom'])
+            if 'haw' in context.op_config:
+                return 'Aloha {whom}!'.format(whom=context.op_config['haw']['whom'])
+            if 'cn' in context.op_config:
+                return '你好，{whom}!'.format(whom=context.op_config['cn']['whom'])
+            if 'en' in context.op_config:
+                return 'Hello, {whom}!'.format(whom=context.op_config['en']['whom'])
     """
 
     def __new__(cls, fields, description=None):
@@ -229,27 +253,31 @@ class Selector(_ConfigHasFields):
 # Config syntax expansion code below
 
 
-def is_potential_field(potential_field):
+def is_potential_field(potential_field: object) -> bool:
     from .field import Field, resolve_to_config_type
 
-    return isinstance(potential_field, (Field, dict, list)) or resolve_to_config_type(
-        potential_field
+    return isinstance(potential_field, (Field, dict, list)) or bool(
+        resolve_to_config_type(potential_field)
     )
 
 
-def convert_fields_to_dict_type(fields):
+def convert_fields_to_dict_type(fields: Dict[str, object]):
     return _convert_fields_to_dict_type(fields, fields, [])
 
 
-def _convert_fields_to_dict_type(original_root, fields, stack):
+def _convert_fields_to_dict_type(
+    original_root: object, fields: Dict[str, object], stack: List[str]
+) -> Shape:
     return Shape(_expand_fields_dict(original_root, fields, stack))
 
 
-def expand_fields_dict(fields):
+def expand_fields_dict(fields: Dict[str, object]) -> Dict[str, "Field"]:
     return _expand_fields_dict(fields, fields, [])
 
 
-def _expand_fields_dict(original_root, fields, stack):
+def _expand_fields_dict(
+    original_root: object, fields: Dict[str, object], stack: List[str]
+) -> Dict[str, "Field"]:
     check.dict_param(fields, "fields")
     return {
         name: _convert_potential_field(original_root, value, stack + [name])
@@ -257,8 +285,7 @@ def _expand_fields_dict(original_root, fields, stack):
     }
 
 
-def expand_list(original_root, the_list, stack):
-    from .config_type import Array
+def expand_list(original_root: object, the_list: List[object], stack: List[str]) -> Array:
 
     if len(the_list) != 1:
         raise DagsterInvalidConfigDefinitionError(
@@ -279,11 +306,11 @@ def expand_list(original_root, the_list, stack):
     return Array(inner_type)
 
 
-def convert_potential_field(potential_field):
+def convert_potential_field(potential_field: object) -> "Field":
     return _convert_potential_field(potential_field, potential_field, [])
 
 
-def _convert_potential_type(original_root, potential_type, stack):
+def _convert_potential_type(original_root: object, potential_type, stack: List[str]):
     from .field import resolve_to_config_type
 
     if isinstance(potential_type, dict):
@@ -295,7 +322,9 @@ def _convert_potential_type(original_root, potential_type, stack):
     return resolve_to_config_type(potential_type)
 
 
-def _convert_potential_field(original_root, potential_field, stack):
+def _convert_potential_field(
+    original_root: object, potential_field: object, stack: List[str]
+) -> "Field":
     from .field import Field
 
     if potential_field is None:
