@@ -2,11 +2,12 @@ from functools import update_wrapper
 from typing import Any, Callable, Dict, List, Optional, Union
 
 from dagster import check
+from dagster.core.decorator_utils import format_docstring_for_description
 
-from ..composition import do_composition
+from ..composition import do_composition, get_validated_config_mapping
 from ..input import InputDefinition
 from ..output import OutputDefinition
-from ..solid import CompositeSolidDefinition
+from ..solid_definition import CompositeSolidDefinition
 
 
 class _CompositeSolid:
@@ -33,6 +34,10 @@ class _CompositeSolid:
         if not self.name:
             self.name = fn.__name__
 
+        config_mapping = get_validated_config_mapping(
+            self.name, self.config_schema, self.config_fn, decorator_name="composite_solid"
+        )
+
         (
             input_mappings,
             output_mappings,
@@ -46,8 +51,7 @@ class _CompositeSolid:
             fn,
             self.input_defs,
             self.output_defs,
-            self.config_schema,
-            self.config_fn,
+            config_mapping,
             ignore_output_from_composition_fn=False,
         )
 
@@ -57,7 +61,7 @@ class _CompositeSolid:
             output_mappings=output_mappings,
             dependencies=dependencies,
             solid_defs=solid_defs,
-            description=self.description or fn.__doc__,
+            description=self.description or format_docstring_for_description(fn),
             config_mapping=config_mapping,
             positional_inputs=positional_inputs,
         )
