@@ -1,28 +1,46 @@
 from functools import update_wrapper
-from typing import TYPE_CHECKING, Any, Callable, List, Optional, Set, Union, cast
+from typing import TYPE_CHECKING, AbstractSet, Any, Callable, List, Optional, Union, cast
 
 from dagster import check
 from dagster.core.errors import DagsterInvalidDefinitionError
 
-from ...decorator_utils import split_function_parameters, validate_decorated_fn_positionals
+from ...decorator_utils import get_function_params, validate_expected_params
 from ..events import HookExecutionResult
-from ..hook import HookDefinition
+from ..hook_definition import HookDefinition
 
 if TYPE_CHECKING:
-    from dagster.core.execution.context.system import HookContext
+    from dagster.core.execution.context.hook import HookContext
     from dagster.core.events import DagsterEvent
 
 
+def _validate_hook_fn_params(fn, expected_positionals):
+    params = get_function_params(fn)
+    missing_positional = validate_expected_params(params, expected_positionals)
+    if missing_positional:
+        raise DagsterInvalidDefinitionError(
+            "'{hook_name}' decorated function does not have required positional "
+            "parameter '{missing_param}'. Hook functions should only have keyword arguments "
+            "that match input names and a first positional parameter named 'context' and "
+            "a second positional parameter named 'event_list'.".format(
+                hook_name=fn.__name__, missing_param=missing_positional
+            )
+        )
+
+
 class _Hook:
     def __init__(
-        self, name: Optional[str] = None, required_resource_keys: Optional[Set[str]] = None
+        self,
+        name: Optional[str] = None,
+        required_resource_keys: Optional[AbstractSet[str]] = None,
+        decorated_fn: Optional[Callable[..., Any]] = None,
     ):
         self.name = check.opt_str_param(name, "name")
         self.required_resource_keys = check.opt_set_param(
             required_resource_keys, "required_resource_keys"
         )
+        self.decorated_fn = check.opt_callable_param(decorated_fn, "decorated_fn")
 
-    def __call__(self, fn: Callable[["HookContext", List["DagsterEvent"]], Any]) -> HookDefinition:
+    def __call__(self, fn) -> HookDefinition:
 
         check.callable_param(fn, "fn")
 
@@ -30,22 +48,14 @@ class _Hook:
             self.name = fn.__name__
 
         expected_positionals = ["context", "event_list"]
-        fn_positionals, _ = split_function_parameters(fn, expected_positionals)
-        missing_positional = validate_decorated_fn_positionals(fn_positionals, expected_positionals)
-        if missing_positional:
-            raise DagsterInvalidDefinitionError(
-                "'{hook_name}' decorated function does not have required positional "
-                "parameter '{missing_param}'. Hook functions should only have keyword arguments "
-                "that match input names and a first positional parameter named 'context' and "
-                "a second positional parameter named 'event_list'.".format(
-                    hook_name=fn.__name__, missing_param=missing_positional
-                )
-            )
+
+        _validate_hook_fn_params(fn, expected_positionals)
 
         hook_def = HookDefinition(
-            name=self.name,
+            name=self.name or "",
             hook_fn=fn,
             required_resource_keys=self.required_resource_keys,
+            decorated_fn=self.decorated_fn or fn,
         )
         update_wrapper(cast(Callable[..., Any], hook_def), fn)
         return hook_def
@@ -53,7 +63,8 @@ class _Hook:
 
 def event_list_hook(
     name: Union[Optional[str], Callable[..., Any]] = None,
-    required_resource_keys: Optional[Set[str]] = None,
+    required_resource_keys: Optional[AbstractSet[str]] = None,
+    decorated_fn: Optional[Callable[..., Any]] = None,
 ) -> Union[HookDefinition, _Hook]:
     """Create a generic hook with the specified parameters from the decorated function.
 
@@ -70,7 +81,7 @@ def event_list_hook(
 
     Args:
         name (Optional[str]): The name of this hook.
-        required_resource_keys (Optional[Set[str]]): Keys for the resources required by the
+        required_resource_keys (Optional[AbstractSet[str]]): Keys for the resources required by the
             hook.
 
     Examples:
@@ -96,21 +107,20 @@ def event_list_hook(
         check.invariant(required_resource_keys is None)
         return _Hook()(name)
 
-    return _Hook(name=name, required_resource_keys=required_resource_keys)
+    return _Hook(
+        name=name, required_resource_keys=required_resource_keys, decorated_fn=decorated_fn
+    )
 
 
 def success_hook(
     name: Union[Optional[str], Callable[..., Any]] = None,
-    required_resource_keys: Optional[Set[str]] = None,
-) -> Union[
-    Union[HookDefinition, _Hook],
-    Callable[[Callable[["HookContext"], Any]], Union[HookDefinition, _Hook]],
-]:
+    required_resource_keys: Optional[AbstractSet[str]] = None,
+) -> Union[Union[HookDefinition, _Hook], Callable[..., Union[HookDefinition, _Hook]],]:
     """Create a hook on step success events with the specified parameters from the decorated function.
 
     Args:
         name (Optional[str]): The name of this hook.
-        required_resource_keys (Optional[Set[str]]): Keys for the resources required by the
+        required_resource_keys (Optional[AbstractSet[str]]): Keys for the resources required by the
             hook.
 
     Examples:
@@ -119,7 +129,7 @@ def success_hook(
 
             @success_hook(required_resource_keys={'slack'})
             def slack_message_on_success(context):
-                message = 'solid {} succeeded'.format(context.solid.name)
+                message = 'op {} succeeded'.format(context.op.name)
                 context.resources.slack.send_message(message)
 
             @success_hook
@@ -129,28 +139,19 @@ def success_hook(
 
     """
 
-    def wrapper(fn: Callable[["HookContext"], Any]) -> Union[HookDefinition, _Hook]:
+    def wrapper(fn) -> Union[HookDefinition, _Hook]:
 
         check.callable_param(fn, "fn")
 
         expected_positionals = ["context"]
-        fn_positionals, _ = split_function_parameters(fn, expected_positionals)
-        missing_positional = validate_decorated_fn_positionals(fn_positionals, expected_positionals)
-        if missing_positional:
-            raise DagsterInvalidDefinitionError(
-                "@success_hook '{hook_name}' decorated function does not have required positional "
-                "parameter '{missing_param}'. Hook functions should only have keyword arguments "
-                "that match input names and a first positional parameter named 'context'.".format(
-                    hook_name=fn.__name__, missing_param=missing_positional
-                )
-            )
+        _validate_hook_fn_params(fn, expected_positionals)
 
         if name is None or callable(name):
             _name = fn.__name__
         else:
             _name = name
 
-        @event_list_hook(_name, required_resource_keys)
+        @event_list_hook(_name, required_resource_keys, decorated_fn=fn)
         def _success_hook(
             context: "HookContext", event_list: List["DagsterEvent"]
         ) -> HookExecutionResult:
@@ -173,7 +174,7 @@ def success_hook(
 
 
 def failure_hook(
-    name: Optional[str] = None, required_resource_keys: Optional[Set[str]] = None
+    name: Optional[str] = None, required_resource_keys: Optional[AbstractSet[str]] = None
 ) -> Union[
     Union[HookDefinition, _Hook],
     Callable[[Callable[["HookContext"], Any]], Union[HookDefinition, _Hook]],
@@ -182,7 +183,7 @@ def failure_hook(
 
     Args:
         name (Optional[str]): The name of this hook.
-        required_resource_keys (Optional[Set[str]]): Keys for the resources required by the
+        required_resource_keys (Optional[AbstractSet[str]]): Keys for the resources required by the
             hook.
 
     Examples:
@@ -191,7 +192,7 @@ def failure_hook(
 
             @failure_hook(required_resource_keys={'slack'})
             def slack_message_on_failure(context):
-                message = 'solid {} failed'.format(context.solid.name)
+                message = 'op {} failed'.format(context.op.name)
                 context.resources.slack.send_message(message)
 
             @failure_hook
@@ -205,23 +206,14 @@ def failure_hook(
         check.callable_param(fn, "fn")
 
         expected_positionals = ["context"]
-        fn_positionals, _ = split_function_parameters(fn, expected_positionals)
-        missing_positional = validate_decorated_fn_positionals(fn_positionals, expected_positionals)
-        if missing_positional:
-            raise DagsterInvalidDefinitionError(
-                "@failure_hook '{hook_name}' decorated function does not have required positional "
-                "parameter '{missing_param}'. Hook functions should only have keyword arguments "
-                "that match input names and a first positional parameter named 'context'.".format(
-                    hook_name=fn.__name__, missing_param=missing_positional
-                )
-            )
+        _validate_hook_fn_params(fn, expected_positionals)
 
         if name is None or callable(name):
             _name = fn.__name__
         else:
             _name = name
 
-        @event_list_hook(_name, required_resource_keys)
+        @event_list_hook(_name, required_resource_keys, decorated_fn=fn)
         def _failure_hook(
             context: "HookContext", event_list: List["DagsterEvent"]
         ) -> HookExecutionResult:
