@@ -1,17 +1,33 @@
+import copy
 import datetime
 import warnings
 from functools import update_wrapper
-from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, cast
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    Generator,
+    List,
+    NamedTuple,
+    Optional,
+    Union,
+    cast,
+)
 
 from dagster import check
 from dagster.core.definitions.partition import (
     PartitionScheduleDefinition,
     PartitionSetDefinition,
+    ScheduleTimeBasedPartitionsDefinition,
     ScheduleType,
-    TimeBasedPartitionParams,
 )
-from dagster.core.definitions.pipeline import PipelineDefinition
-from dagster.core.errors import DagsterInvalidDefinitionError
+from dagster.core.errors import (
+    DagsterInvalidDefinitionError,
+    ScheduleExecutionError,
+    user_code_error_boundary,
+)
+from dagster.utils import ensure_gen
 from dagster.utils.partitions import (
     DEFAULT_DATE_FORMAT,
     DEFAULT_HOURLY_FORMAT_WITHOUT_TIMEZONE,
@@ -20,8 +36,13 @@ from dagster.utils.partitions import (
     create_offset_partition_selector,
 )
 
+from ...decorator_utils import get_function_params
+from ...storage.tags import check_tags
+from ..graph_definition import GraphDefinition
 from ..mode import DEFAULT_MODE_NAME
-from ..schedule import ScheduleDefinition
+from ..pipeline_definition import PipelineDefinition
+from ..run_request import RunRequest, SkipReason
+from ..schedule_definition import ScheduleDefinition, is_context_provided
 
 if TYPE_CHECKING:
     from dagster import ScheduleEvaluationContext, Partition
@@ -29,12 +50,24 @@ if TYPE_CHECKING:
 # Error messages are long
 # pylint: disable=C0301
 
+RunConfig = Dict[str, Any]
+RunRequestGenerator = Generator[Union[RunRequest, SkipReason], None, None]
+
+
+class DecoratedScheduleFunction(NamedTuple):
+    """Wrapper around the decorated schedule function.  Keeps track of both to better support the
+    optimal return value for direct invocation of the evaluation function"""
+
+    decorated_fn: Callable[..., Union[RunRequest, SkipReason, RunConfig, RunRequestGenerator]]
+    wrapped_fn: Callable[["ScheduleEvaluationContext"], RunRequestGenerator]
+    has_context_arg: bool
+
 
 def schedule(
     cron_schedule: str,
     pipeline_name: Optional[str] = None,
     name: Optional[str] = None,
-    tags: Optional[Dict[str, Any]] = None,
+    tags: Optional[Dict[str, str]] = None,
     tags_fn: Optional[Callable[["ScheduleEvaluationContext"], Optional[Dict[str, str]]]] = None,
     solid_selection: Optional[List[str]] = None,
     mode: Optional[str] = "default",
@@ -42,19 +75,36 @@ def schedule(
     environment_vars: Optional[Dict[str, str]] = None,
     execution_timezone: Optional[str] = None,
     description: Optional[str] = None,
-    job: Optional[PipelineDefinition] = None,
-) -> Callable[[Callable[["ScheduleEvaluationContext"], Dict[str, Any]]], ScheduleDefinition]:
-    """Create a schedule.
+    job: Optional[Union[PipelineDefinition, GraphDefinition]] = None,
+) -> Callable[
+    [
+        Callable[
+            ...,
+            Union[RunRequest, SkipReason, RunConfig, RunRequestGenerator],
+        ]
+    ],
+    ScheduleDefinition,
+]:
+    """
+    Creates a schedule following the provided cron schedule and requests runs for the provided job.
 
-    The decorated function will be called as the ``run_config_fn`` of the underlying
-    :py:class:`~dagster.ScheduleDefinition` and should take a
-    :py:class:`~dagster.ScheduleEvaluationContext` as its only argument, returning the run config
-    for the scheduled execution.
+    The decorated function takes in a :py:class:`~dagster.ScheduleEvaluationContext` as its only
+    argument, and does one of the following:
+
+    1. Return a `RunRequest` object.
+    2. Yield multiple of `RunRequest` objects.
+    3. Return or yield a `SkipReason` object, providing a descriptive message of why no runs were
+       requested.
+    4. Return or yield nothing (skipping without providing a reason)
+    5. Return a run config dictionary.
+
+    Returns a :py:class:`~dagster.ScheduleDefinition`.
 
     Args:
         cron_schedule (str): A valid cron string specifying when the schedule will run, e.g.,
             ``'45 23 * * 6'`` for a schedule that runs at 11:45 PM every Saturday.
-        pipeline_name (str): The name of the pipeline to execute when the schedule runs.
+        pipeline_name (Optional[str]): (legacy)  The name of the pipeline to execute when the
+            schedule runs.
         name (Optional[str]): The name of the schedule to create.
         tags (Optional[Dict[str, str]]): A dictionary of tags (string key-value pairs) to attach
             to the scheduled runs.
@@ -72,30 +122,88 @@ def schedule(
             schedule should execute). Defaults to a function that always returns ``True``.
         environment_vars (Optional[Dict[str, str]]): Any environment variables to set when executing
             the schedule.
-        execution_timezone (Optional[str]): Timezone in which the schedule should run. Only works
-            with DagsterDaemonScheduler, and must be set when using that scheduler.
+        execution_timezone (Optional[str]): Timezone in which the schedule should run.
+            Supported strings for timezones are the ones provided by the
+            `IANA time zone database <https://www.iana.org/time-zones>` - e.g. "America/Los_Angeles".
         description (Optional[str]): A human-readable description of the schedule.
-        job (Optional[PipelineDefinition]): Experimental
+        job (Optional[Union[GraphDefinition, JobDefinition]]): The job that should execute when this
+            schedule runs.
     """
 
-    def inner(fn: Callable[["ScheduleEvaluationContext"], Dict[str, Any]]) -> ScheduleDefinition:
+    def inner(
+        fn: Callable[
+            ...,
+            Union[RunRequest, SkipReason, RunConfig, RunRequestGenerator],
+        ]
+    ) -> ScheduleDefinition:
         check.callable_param(fn, "fn")
 
         schedule_name = name or fn.__name__
 
+        # perform upfront validation of schedule tags
+        _tags_fn: Optional[Callable[["ScheduleEvaluationContext"], Dict[str, str]]] = None
+        if tags_fn and tags:
+            raise DagsterInvalidDefinitionError(
+                "Attempted to provide both tags_fn and tags as arguments"
+                " to ScheduleDefinition. Must provide only one of the two."
+            )
+        elif tags:
+            check_tags(tags, "tags")
+            _tags_fn = cast(Callable[["ScheduleEvaluationContext"], Dict[str, str]], lambda _: tags)
+        elif tags_fn:
+            _tags_fn = cast(
+                Callable[["ScheduleEvaluationContext"], Dict[str, str]],
+                lambda context: tags_fn(context) or {},
+            )
+
+        def _wrapped_fn(context: "ScheduleEvaluationContext"):
+            if should_execute:
+                with user_code_error_boundary(
+                    ScheduleExecutionError,
+                    lambda: f"Error occurred during the execution of should_execute for schedule {schedule_name}",
+                ):
+                    if not should_execute(context):
+                        yield SkipReason(
+                            f"should_execute function for {schedule_name} returned false."
+                        )
+                        return
+
+            with user_code_error_boundary(
+                ScheduleExecutionError,
+                lambda: f"Error occurred during the evaluation of schedule {schedule_name}",
+            ):
+                result = fn(context) if has_context_arg else fn()
+                if isinstance(result, dict):
+                    # this is the run-config based decorated function, wrap the evaluated run config
+                    # and tags in a RunRequest
+                    evaluated_run_config = copy.deepcopy(result)
+                    evaluated_tags = _tags_fn(context) if _tags_fn else None
+                    yield RunRequest(
+                        run_key=None,
+                        run_config=evaluated_run_config,
+                        tags=evaluated_tags,
+                    )
+                else:
+                    # this is a run-request based decorated function
+                    yield from ensure_gen(result)
+
+        has_context_arg = is_context_provided(get_function_params(fn))
+        evaluation_fn = DecoratedScheduleFunction(
+            decorated_fn=fn,
+            wrapped_fn=_wrapped_fn,
+            has_context_arg=has_context_arg,
+        )
+
         schedule_def = ScheduleDefinition(
             name=schedule_name,
             cron_schedule=cron_schedule,
             pipeline_name=pipeline_name,
-            run_config_fn=fn,
-            tags=tags,
-            tags_fn=tags_fn,
             solid_selection=solid_selection,
             mode=mode,
-            should_execute=should_execute,
             environment_vars=environment_vars,
             execution_timezone=execution_timezone,
             description=description,
+            execution_fn=evaluation_fn,
             job=job,
         )
 
@@ -121,7 +229,6 @@ def monthly_schedule(
     execution_timezone: Optional[str] = None,
     partition_months_offset: Optional[int] = 1,
     description: Optional[str] = None,
-    job: Optional[PipelineDefinition] = None,
 ) -> Callable[[Callable[[datetime.datetime], Dict[str, Any]]], PartitionScheduleDefinition]:
     """Create a partitioned schedule that runs monthly.
 
@@ -155,14 +262,14 @@ def monthly_schedule(
             the schedule.
         end_date (Optional[datetime.datetime]): The last time to run the schedule to, defaults to
             current time.
-        execution_timezone (Optional[str]): Timezone in which the schedule should run. Only works
-            with DagsterDaemonScheduler, and must be set when using that scheduler.
+        execution_timezone (Optional[str]): Timezone in which the schedule should run.
+            Supported strings for timezones are the ones provided by the
+            `IANA time zone database <https://www.iana.org/time-zones>` - e.g. "America/Los_Angeles".
         partition_months_offset (Optional[int]): How many months back to go when choosing the partition
             for a given schedule execution. For example, when partition_months_offset=1, the schedule
             that executes during month N will fill in the partition for month N-1.
             (Default: 1)
         description (Optional[str]): A human-readable description of the schedule.
-        job (Optional[PipelineDefinition]): Experimental
     """
     check.opt_str_param(name, "name")
     check.inst_param(start_date, "start_date", datetime.datetime)
@@ -223,7 +330,7 @@ def my_schedule_definition(_):
 
         fmt = DEFAULT_MONTHLY_FORMAT
 
-        partition_params = TimeBasedPartitionParams(
+        partitions_def = ScheduleTimeBasedPartitionsDefinition(
             schedule_type=ScheduleType.MONTHLY,
             start=start_date,
             execution_day=execution_day_of_month,
@@ -236,26 +343,25 @@ def my_schedule_definition(_):
 
         partition_set = PartitionSetDefinition(
             name="{}_partitions".format(schedule_name),
-            pipeline_name=pipeline_name,
+            pipeline_name=pipeline_name,  # type: ignore[arg-type]
             run_config_fn_for_partition=lambda partition: fn(partition.value),
             solid_selection=solid_selection,
             tags_fn_for_partition=tags_fn_for_partition_value,
             mode=mode,
-            partition_params=partition_params,
+            partitions_def=partitions_def,
         )
 
         schedule_def = partition_set.create_schedule_definition(
             schedule_name,
-            partition_params.get_cron_schedule(),
+            partitions_def.get_cron_schedule(),
             should_execute=should_execute,
             environment_vars=environment_vars,
             partition_selector=create_offset_partition_selector(
-                execution_time_to_partition_fn=partition_params.get_execution_time_to_partition_fn()
+                execution_time_to_partition_fn=partitions_def.get_execution_time_to_partition_fn()
             ),
             execution_timezone=execution_timezone,
             description=description,
             decorated_fn=fn,
-            job=job,
         )
         update_wrapper(schedule_def, wrapped=fn)
 
@@ -279,7 +385,6 @@ def weekly_schedule(
     execution_timezone: Optional[str] = None,
     partition_weeks_offset: Optional[int] = 1,
     description: Optional[str] = None,
-    job: Optional[PipelineDefinition] = None,
 ) -> Callable[[Callable[[datetime.datetime], Dict[str, Any]]], PartitionScheduleDefinition]:
     """Create a partitioned schedule that runs daily.
 
@@ -290,6 +395,7 @@ def weekly_schedule(
     configuration for the scheduled run.
 
     The decorator produces a :py:class:`~dagster.PartitionScheduleDefinition`.
+
     Args:
         pipeline_name (str): The name of the pipeline to execute when the schedule runs.
         start_date (datetime.datetime): The date from which to run the schedule.
@@ -312,14 +418,14 @@ def weekly_schedule(
             the schedule.
         end_date (Optional[datetime.datetime]): The last time to run the schedule to, defaults to
             current time.
-        execution_timezone (Optional[str]): Timezone in which the schedule should run. Only works
-            with DagsterDaemonScheduler, and must be set when using that scheduler.
+        execution_timezone (Optional[str]): Timezone in which the schedule should run.
+            Supported strings for timezones are the ones provided by the
+            `IANA time zone database <https://www.iana.org/time-zones>` - e.g. "America/Los_Angeles".
         partition_weeks_offset (Optional[int]): How many weeks back to go when choosing the partition
             for a given schedule execution. For example, when partition_weeks_offset=1, the schedule
             that executes during week N will fill in the partition for week N-1.
             (Default: 1)
         description (Optional[str]): A human-readable description of the schedule.
-        job (Optional[PipelineDefinition]): Experimental
     """
     check.opt_str_param(name, "name")
     check.inst_param(start_date, "start_date", datetime.datetime)
@@ -375,7 +481,7 @@ def my_schedule_definition(_):
 
         fmt = DEFAULT_DATE_FORMAT
 
-        partition_params = TimeBasedPartitionParams(
+        partitions_def = ScheduleTimeBasedPartitionsDefinition(
             schedule_type=ScheduleType.WEEKLY,
             start=start_date,
             execution_time=execution_time,
@@ -388,26 +494,25 @@ def my_schedule_definition(_):
 
         partition_set = PartitionSetDefinition(
             name="{}_partitions".format(schedule_name),
-            pipeline_name=pipeline_name,
+            pipeline_name=pipeline_name,  # type: ignore[arg-type]
             run_config_fn_for_partition=lambda partition: fn(partition.value),
             solid_selection=solid_selection,
             tags_fn_for_partition=tags_fn_for_partition_value,
             mode=mode,
-            partition_params=partition_params,
+            partitions_def=partitions_def,
         )
 
         schedule_def = partition_set.create_schedule_definition(
             schedule_name,
-            partition_params.get_cron_schedule(),
+            partitions_def.get_cron_schedule(),
             should_execute=should_execute,
             environment_vars=environment_vars,
             partition_selector=create_offset_partition_selector(
-                execution_time_to_partition_fn=partition_params.get_execution_time_to_partition_fn(),
+                execution_time_to_partition_fn=partitions_def.get_execution_time_to_partition_fn(),
             ),
             execution_timezone=execution_timezone,
             description=description,
             decorated_fn=fn,
-            job=job,
         )
 
         update_wrapper(schedule_def, wrapped=fn)
@@ -430,7 +535,6 @@ def daily_schedule(
     execution_timezone: Optional[str] = None,
     partition_days_offset: Optional[int] = 1,
     description: Optional[str] = None,
-    job: Optional[PipelineDefinition] = None,
 ) -> Callable[[Callable[[datetime.datetime], Dict[str, Any]]], PartitionScheduleDefinition]:
     """Create a partitioned schedule that runs daily.
 
@@ -462,14 +566,14 @@ def daily_schedule(
             the schedule.
         end_date (Optional[datetime.datetime]): The last time to run the schedule to, defaults to
             current time.
-        execution_timezone (Optional[str]): Timezone in which the schedule should run. Only works
-            with DagsterDaemonScheduler, and must be set when using that scheduler.
+        execution_timezone (Optional[str]): Timezone in which the schedule should run.
+            Supported strings for timezones are the ones provided by the
+            `IANA time zone database <https://www.iana.org/time-zones>` - e.g. "America/Los_Angeles".
         partition_days_offset (Optional[int]): How many days back to go when choosing the partition
             for a given schedule execution. For example, when partition_days_offset=1, the schedule
             that executes during day N will fill in the partition for day N-1.
             (Default: 1)
         description (Optional[str]): A human-readable description of the schedule.
-        job (Optional[PipelineDefinition]): Experimental
     """
     check.opt_str_param(pipeline_name, "pipeline_name")
     check.inst_param(start_date, "start_date", datetime.datetime)
@@ -517,7 +621,7 @@ def my_schedule_definition(_):
             )
             tags_fn_for_partition_value = lambda partition: tags_fn(partition.value)
 
-        partition_params = TimeBasedPartitionParams(
+        partitions_def = ScheduleTimeBasedPartitionsDefinition(
             schedule_type=ScheduleType.DAILY,
             start=start_date,
             execution_time=execution_time,
@@ -529,26 +633,25 @@ def my_schedule_definition(_):
 
         partition_set = PartitionSetDefinition(
             name="{}_partitions".format(schedule_name),
-            pipeline_name=pipeline_name,
+            pipeline_name=pipeline_name,  # type: ignore[arg-type]
             run_config_fn_for_partition=lambda partition: fn(partition.value),
             solid_selection=solid_selection,
             tags_fn_for_partition=tags_fn_for_partition_value,
             mode=mode,
-            partition_params=partition_params,
+            partitions_def=partitions_def,
         )
 
         schedule_def = partition_set.create_schedule_definition(
             schedule_name,
-            partition_params.get_cron_schedule(),
+            partitions_def.get_cron_schedule(),
             should_execute=should_execute,
             environment_vars=environment_vars,
             partition_selector=create_offset_partition_selector(
-                execution_time_to_partition_fn=partition_params.get_execution_time_to_partition_fn(),
+                execution_time_to_partition_fn=partitions_def.get_execution_time_to_partition_fn(),
             ),
             execution_timezone=execution_timezone,
             description=description,
             decorated_fn=fn,
-            job=job,
         )
         update_wrapper(schedule_def, wrapped=fn)
         return schedule_def
@@ -570,7 +673,6 @@ def hourly_schedule(
     execution_timezone: Optional[str] = None,
     partition_hours_offset: Optional[int] = 1,
     description: Optional[str] = None,
-    job: Optional[PipelineDefinition] = None,
 ) -> Callable[[Callable[[datetime.datetime], Dict[str, Any]]], PartitionScheduleDefinition]:
     """Create a partitioned schedule that runs hourly.
 
@@ -604,14 +706,14 @@ def hourly_schedule(
             the schedule.
         end_date (Optional[datetime.datetime]): The last time to run the schedule to, defaults to
             current time.
-        execution_timezone (Optional[str]): Timezone in which the schedule should run. Only works
-            with DagsterDaemonScheduler, and must be set when using that scheduler.
+        execution_timezone (Optional[str]): Timezone in which the schedule should run.
+            Supported strings for timezones are the ones provided by the
+            `IANA time zone database <https://www.iana.org/time-zones>` - e.g. "America/Los_Angeles".
         partition_hours_offset (Optional[int]): How many hours back to go when choosing the partition
             for a given schedule execution. For example, when partition_hours_offset=1, the schedule
             that executes during hour N will fill in the partition for hour N-1.
             (Default: 1)
         description (Optional[str]): A human-readable description of the schedule.
-        job (Optional[PipelineDefinition]): Experimental
     """
     check.opt_str_param(name, "name")
     check.inst_param(start_date, "start_date", datetime.datetime)
@@ -673,7 +775,7 @@ def my_schedule_definition(_):
             else DEFAULT_HOURLY_FORMAT_WITHOUT_TIMEZONE
         )
 
-        partition_params = TimeBasedPartitionParams(
+        partitions_def = ScheduleTimeBasedPartitionsDefinition(
             schedule_type=ScheduleType.HOURLY,
             start=start_date,
             execution_time=execution_time,
@@ -685,26 +787,25 @@ def my_schedule_definition(_):
 
         partition_set = PartitionSetDefinition(
             name="{}_partitions".format(schedule_name),
-            pipeline_name=pipeline_name,
+            pipeline_name=pipeline_name,  # type: ignore[arg-type]
             run_config_fn_for_partition=lambda partition: fn(partition.value),
             solid_selection=solid_selection,
             tags_fn_for_partition=tags_fn_for_partition_value,
             mode=mode,
-            partition_params=partition_params,
+            partitions_def=partitions_def,
         )
 
         schedule_def = partition_set.create_schedule_definition(
             schedule_name,
-            partition_params.get_cron_schedule(),
+            partitions_def.get_cron_schedule(),
             should_execute=should_execute,
             environment_vars=environment_vars,
             partition_selector=create_offset_partition_selector(
-                execution_time_to_partition_fn=partition_params.get_execution_time_to_partition_fn(),
+                execution_time_to_partition_fn=partitions_def.get_execution_time_to_partition_fn(),
             ),
             execution_timezone=execution_timezone,
             description=description,
             decorated_fn=fn,
-            job=job,
         )
 
         update_wrapper(schedule_def, wrapped=fn)
