@@ -2,6 +2,7 @@ from functools import lru_cache, update_wrapper
 from typing import Any, Callable, Dict, List, NamedTuple, Optional, Sequence, Set, Union, cast
 
 from dagster import check
+from dagster.core.decorator_utils import format_docstring_for_description
 from dagster.core.errors import DagsterInvalidDefinitionError
 from dagster.core.types.dagster_type import DagsterTypeKind
 from dagster.seven import funcsigs
@@ -10,13 +11,12 @@ from ...decorator_utils import (
     get_function_params,
     get_valid_name_permutations,
     positional_arg_name_list,
-    validate_expected_params,
 )
 from ..inference import infer_input_props, infer_output_props
 from ..input import InputDefinition
 from ..output import OutputDefinition
 from ..policy import RetryPolicy
-from ..solid import SolidDefinition
+from ..solid_definition import SolidDefinition
 
 
 class DecoratedSolidFunction(NamedTuple):
@@ -103,7 +103,6 @@ class _Solid:
             fn_name=self.name,
             compute_fn=compute_fn,
             explicit_input_defs=self.input_defs,
-            context_required=bool(self.config_schema),
             exclude_nothing=True,
         )
 
@@ -113,7 +112,7 @@ class _Solid:
             output_defs=output_defs,
             compute_fn=compute_fn,
             config_schema=self.config_schema,
-            description=self.description or fn.__doc__,
+            description=self.description or format_docstring_for_description(fn),
             required_resource_keys=self.required_resource_keys,
             tags=self.tags,
             version=self.version,
@@ -264,7 +263,6 @@ def resolve_checked_solid_fn_inputs(
     fn_name: str,
     compute_fn: DecoratedSolidFunction,
     explicit_input_defs: List[InputDefinition],
-    context_required: bool,
     exclude_nothing: bool,
 ) -> List[InputDefinition]:
     """
@@ -278,14 +276,10 @@ def resolve_checked_solid_fn_inputs(
             DecoratedSolidFunction wrapper.
         explicit_input_defs (List[InputDefinition]): The input definitions that were explicitly
             provided in the decorator.
-        context_required (bool): True if a context argument is required due to environment
-            information being provided in the decorator (ie: resources, config).
         exclude_nothing (bool): True if Nothing type inputs should be excluded from compute_fn
             arguments.
     """
 
-    expected_positionals = ["context"] if context_required or compute_fn.has_context_arg() else []
-
     if exclude_nothing:
         explicit_names = set(
             inp.name
@@ -303,23 +297,7 @@ def resolve_checked_solid_fn_inputs(
 
     params = get_function_params(compute_fn.decorated_fn)
 
-    if context_required:
-
-        missing_param = validate_expected_params(params, expected_positionals)
-
-        if missing_param:
-            raise DagsterInvalidDefinitionError(
-                "{decorator_name} '{solid_name}' decorated function requires positional parameter "
-                "'{missing_param}', but it was not provided. '{missing_param}' is required because "
-                "either 'required_resource_keys' or 'config_schema' was specified.".format(
-                    decorator_name=decorator_name, solid_name=fn_name, missing_param=missing_param
-                )
-            )
-
-        input_args = params[len(expected_positionals) :]
-
-    else:
-        input_args = params[1:] if compute_fn.has_context_arg() else params
+    input_args = params[1:] if compute_fn.has_context_arg() else params
 
     # Validate input arguments
     used_inputs = set()
@@ -332,9 +310,9 @@ def resolve_checked_solid_fn_inputs(
         elif param.kind == funcsigs.Parameter.VAR_POSITIONAL:
             raise DagsterInvalidDefinitionError(
                 f"{decorator_name} '{fn_name}' decorated function has positional vararg parameter "
-                f"'{param}'. Solid functions should only have keyword arguments that match "
-                "input names and, if system information is required, a first positional "
-                "parameter named 'context'."
+                f"'{param}'. {decorator_name} decorated functions should only have keyword "
+                "arguments that match input names and, if system information is required, a first "
+                "positional parameter named 'context'."
             )
 
         else:
@@ -342,7 +320,7 @@ def resolve_checked_solid_fn_inputs(
                 if param.name in nothing_names:
                     raise DagsterInvalidDefinitionError(
                         f"{decorator_name} '{fn_name}' decorated function has parameter '{param.name}' that is "
-                        "one of the solid input_defs of type 'Nothing' which should not be included since "
+                        "one of the input_defs of type 'Nothing' which should not be included since "
                         "no data will be passed for it. "
                     )
                 else:
@@ -356,9 +334,9 @@ def resolve_checked_solid_fn_inputs(
         undeclared_inputs_printed = ", '".join(undeclared_inputs)
         raise DagsterInvalidDefinitionError(
             f"{decorator_name} '{fn_name}' decorated function does not have parameter(s) "
-            f"'{undeclared_inputs_printed}', which are in solid's input_defs. Solid functions "
-            "should only have keyword arguments that match input names and, if system information "
-            "is required, a first positional parameter named 'context'."
+            f"'{undeclared_inputs_printed}', which are in provided input_defs. {decorator_name} "
+            "decorated functions should only have keyword arguments that match input names and, if "
+            "system information is required, a first positional parameter named 'context'."
         )
 
     inferred_props = {
