@@ -50,8 +50,8 @@ class DagsterInvalidDefinitionError(DagsterError):
 
 class DagsterInvalidSubsetError(DagsterError):
     """Indicates that a subset of a pipeline is invalid because either:
-    - One or more solids in the specified subset do not exist on the pipeline.'
-    - The subset produces an invalid pipeline.
+    - One or more ops in the specified subset do not exist on the job.'
+    - The subset produces an invalid job.
     """
 
 
@@ -153,7 +153,7 @@ def raise_execution_interrupts():
 
 
 @contextmanager
-def user_code_error_boundary(error_cls, msg_fn, **kwargs):
+def user_code_error_boundary(error_cls, msg_fn, log_manager=None, **kwargs):
     """
     Wraps the execution of user-space code in an error boundary. This places a uniform
     policy around any user code invoked by the framework. This ensures that all user
@@ -179,6 +179,8 @@ def user_code_error_boundary(error_cls, msg_fn, **kwargs):
     check.subclass_param(error_cls, "error_cls", DagsterUserCodeExecutionError)
 
     with raise_execution_interrupts():
+        if log_manager:
+            log_manager.begin_python_log_capture()
         try:
             yield
         except DagsterError as de:
@@ -190,6 +192,9 @@ def user_code_error_boundary(error_cls, msg_fn, **kwargs):
             raise error_cls(
                 msg_fn(), user_exception=e, original_exc_info=sys.exc_info(), **kwargs
             ) from e
+        finally:
+            if log_manager:
+                log_manager.end_python_log_capture()
 
 
 class DagsterUserCodeExecutionError(DagsterError):
@@ -224,7 +229,7 @@ class DagsterUserCodeExecutionError(DagsterError):
 
 
 class DagsterTypeCheckError(DagsterUserCodeExecutionError):
-    """Indicates an error in the solid type system at runtime. E.g. a solid receives an
+    """Indicates an error in the op type system at runtime. E.g. a op receives an
     unexpected input, or produces an output that does not match the type of the output definition.
     """
 
@@ -239,7 +244,7 @@ class DagsterExecutionLoadInputError(DagsterUserCodeExecutionError):
 
 
 class DagsterExecutionHandleOutputError(DagsterUserCodeExecutionError):
-    """Indicates an error occurred while loading an input for a step."""
+    """Indicates an error occurred while handling an output for a step."""
 
     def __init__(self, *args, **kwargs):
         self.step_key = check.str_param(kwargs.pop("step_key"), "step_key")
@@ -252,8 +257,8 @@ class DagsterExecutionStepExecutionError(DagsterUserCodeExecutionError):
 
     def __init__(self, *args, **kwargs):
         self.step_key = check.str_param(kwargs.pop("step_key"), "step_key")
-        self.solid_name = check.str_param(kwargs.pop("solid_name"), "solid_name")
-        self.solid_def_name = check.str_param(kwargs.pop("solid_def_name"), "solid_def_name")
+        self.op_name = check.str_param(kwargs.pop("op_name"), "op_name")
+        self.op_def_name = check.str_param(kwargs.pop("op_def_name"), "op_def_name")
         super(DagsterExecutionStepExecutionError, self).__init__(*args, **kwargs)
 
 
@@ -267,7 +272,8 @@ class DagsterResourceFunctionError(DagsterUserCodeExecutionError):
 class DagsterConfigMappingFunctionError(DagsterUserCodeExecutionError):
     """
     Indicates that an unexpected error occurred while executing the body of a config mapping
-    function defined in a :py:class:`~dagster.CompositeSolidDefinition` during config parsing.
+    function defined in a :py:class:`~dagster.JobDefinition` or `~dagster.GraphDefinition` during
+    config parsing.
     """
 
 
@@ -290,8 +296,8 @@ class DagsterUnknownResourceError(DagsterError, AttributeError):
     # inherits from AttributeError as it is raised within a __getattr__ call... used to support
     # object hasattr method
     """Indicates that an unknown resource was accessed in the body of an execution step. May often
-    happen by accessing a resource in the compute function of a solid without first supplying the
-    solid with the correct `required_resource_keys` argument.
+    happen by accessing a resource in the compute function of an op without first supplying the
+    op with the correct `required_resource_keys` argument.
     """
 
     def __init__(self, resource_name, *args, **kwargs):
@@ -305,7 +311,7 @@ class DagsterUnknownResourceError(DagsterError, AttributeError):
 
 class DagsterInvalidInvocationError(DagsterError):
     """
-    Indicates that an error has occurred when a solid has been invoked, but before the actual
+    Indicates that an error has occurred when an op has been invoked, but before the actual
     core compute has been reached.
     """
 
@@ -356,6 +362,10 @@ class DagsterSubprocessError(DagsterError):
         super(DagsterSubprocessError, self).__init__(*args, **kwargs)
 
 
+class DagsterUserCodeUnreachableError(DagsterError):
+    """Dagster was unable to reach a user code server to fetch information about user code."""
+
+
 class DagsterUserCodeProcessError(DagsterError):
     """An exception has occurred in a user code process that the host process raising this error
     was communicating with."""
@@ -420,10 +430,6 @@ class DagsterBackfillFailedError(DagsterError):
         super(DagsterBackfillFailedError, self).__init__(*args, **kwargs)
 
 
-class DagsterScheduleWipeRequired(DagsterError):
-    """Indicates that the user must wipe their stored schedule state."""
-
-
 class DagsterInstanceMigrationRequired(DagsterError):
     """Indicates that the dagster instance must be migrated."""
 
@@ -464,11 +470,10 @@ class DagsterRunConflict(DagsterError):
 class DagsterTypeCheckDidNotPass(DagsterError):
     """Indicates that a type check failed.
 
-    This is raised when ``raise_on_error`` is ``True`` in calls to the synchronous pipeline and
-    solid execution APIs (:py:func:`~dagster.execute_pipeline`, :py:func:`~dagster.execute_solid`,
-    etc.), that is, typically in test, and  a :py:class:`~dagster.DagsterType`'s type check fails
-    by returning either ``False`` or an instance of :py:class:`~dagster.TypeCheck` whose ``success``
-    member is ``False``.
+    This is raised when ``raise_on_error`` is ``True`` in calls to the synchronous job and
+    graph execution APIs (e.g. `graph.execute_in_process()`, `job.execute_in_process()` -- typically
+    within a test), and a :py:class:`~dagster.DagsterType`'s type check fails by returning either
+    ``False`` or an instance of :py:class:`~dagster.TypeCheck` whose ``success`` member is ``False``.
     """
 
     def __init__(self, description=None, metadata_entries=None, dagster_type=None):
@@ -529,7 +534,7 @@ class JobError(DagsterUserCodeExecutionError):
 
 
 class DagsterUnknownStepStateError(DagsterError):
-    """When pipeline execution complete with steps in an unknown state"""
+    """When job execution completes with steps in an unknown state"""
 
 
 class DagsterObjectStoreError(DagsterError):
@@ -546,3 +551,9 @@ class DagsterHomeNotSetError(DagsterError):
     The user has tried to use a command that requires an instance or invoke DagsterInstance.get()
     without setting DAGSTER_HOME env var.
     """
+
+
+class DagsterUnknownPartitionError(DagsterError):
+    """
+    The user has tried to access run config for a partition name that does not exist.
+    """
