@@ -2,7 +2,7 @@ from collections import namedtuple
 from enum import Enum
 
 from dagster import check
-from dagster.core.execution.plan.resume_retry import get_retry_steps_from_execution_plan
+from dagster.core.execution.plan.resume_retry import get_retry_steps_from_parent_run
 from dagster.core.execution.plan.state import KnownExecutionState
 from dagster.core.host_representation import (
     ExternalPartitionSet,
@@ -23,6 +23,7 @@ from dagster.core.storage.tags import (
     RESUME_RETRY_TAG,
     ROOT_RUN_ID_TAG,
 )
+from dagster.core.telemetry import BACKFILL_RUN_CREATED, hash_name, log_action
 from dagster.core.utils import make_new_run_id
 from dagster.core.workspace.workspace import IWorkspace
 from dagster.serdes import whitelist_for_serdes
@@ -37,6 +38,10 @@ class BulkActionStatus(Enum):
     FAILED = "FAILED"
     CANCELED = "CANCELED"
 
+    @staticmethod
+    def from_graphql_input(graphql_str):
+        return BulkActionStatus(graphql_str)
+
 
 @whitelist_for_serdes
 class PartitionBackfill(
@@ -173,6 +178,8 @@ def submit_backfill_runs(instance, workspace, repo_location, backfill_job, parti
 def create_backfill_run(
     instance, repo_location, external_pipeline, external_partition_set, backfill_job, partition_data
 ):
+    from dagster.daemon.daemon import get_telemetry_daemon_session_id
+
     check.inst_param(instance, "instance", DagsterInstance)
     check.inst_param(repo_location, "repo_location", RepositoryLocation)
     check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
@@ -180,14 +187,6 @@ def create_backfill_run(
     check.inst_param(backfill_job, "backfill_job", PartitionBackfill)
     check.inst_param(partition_data, "partition_data", ExternalPartitionExecutionParamData)
 
-    full_external_execution_plan = repo_location.get_external_execution_plan(
-        external_pipeline,
-        partition_data.run_config,
-        external_partition_set.mode,
-        step_keys_to_execute=None,
-        known_state=None,
-    )
-
     tags = merge_dicts(
         external_pipeline.tags,
         partition_data.tags,
@@ -195,11 +194,16 @@ def create_backfill_run(
         backfill_job.tags,
     )
 
+    solids_to_execute = None
+    solid_selection = None
     if not backfill_job.from_failure and not backfill_job.reexecution_steps:
         step_keys_to_execute = None
         parent_run_id = None
         root_run_id = None
         known_state = None
+        if external_partition_set.solid_selection:
+            solids_to_execute = frozenset(external_partition_set.solid_selection)
+            solid_selection = external_partition_set.solid_selection
 
     elif backfill_job.from_failure:
         last_run = _fetch_last_run(instance, external_partition_set, partition_data.name)
@@ -216,9 +220,11 @@ def create_backfill_run(
                 ROOT_RUN_ID_TAG: root_run_id,
             },
         )
-        step_keys_to_execute, known_state = get_retry_steps_from_execution_plan(
-            instance, full_external_execution_plan, parent_run_id
-        )
+        solids_to_execute = last_run.solids_to_execute
+        solid_selection = last_run.solid_selection
+
+        step_keys_to_execute, known_state = get_retry_steps_from_parent_run(instance, parent_run_id)
+
     elif backfill_job.reexecution_steps:
         last_run = _fetch_last_run(instance, external_partition_set, partition_data.name)
         parent_run_id = last_run.run_id if last_run else None
@@ -236,16 +242,26 @@ def create_backfill_run(
         else:
             known_state = None
 
-    if step_keys_to_execute:
-        external_execution_plan = repo_location.get_external_execution_plan(
-            external_pipeline,
-            partition_data.run_config,
-            external_partition_set.mode,
-            step_keys_to_execute=step_keys_to_execute,
-            known_state=known_state,
-        )
-    else:
-        external_execution_plan = full_external_execution_plan
+        if external_partition_set.solid_selection:
+            solids_to_execute = frozenset(external_partition_set.solid_selection)
+            solid_selection = external_partition_set.solid_selection
+
+    external_execution_plan = repo_location.get_external_execution_plan(
+        external_pipeline,
+        partition_data.run_config,
+        external_partition_set.mode,
+        step_keys_to_execute=step_keys_to_execute,
+        known_state=known_state,
+        instance=instance,
+    )
+
+    log_action(
+        instance,
+        BACKFILL_RUN_CREATED,
+        metadata={"DAEMON_SESSION_ID": get_telemetry_daemon_session_id()},
+        repo_hash=hash_name(repo_location.name),
+        pipeline_name_hash=hash_name(external_pipeline.name),
+    )
 
     return instance.create_run(
         pipeline_snapshot=external_pipeline.pipeline_snapshot,
@@ -253,9 +269,7 @@ def create_backfill_run(
         parent_pipeline_snapshot=external_pipeline.parent_pipeline_snapshot,
         pipeline_name=external_pipeline.name,
         run_id=make_new_run_id(),
-        solids_to_execute=frozenset(external_partition_set.solid_selection)
-        if external_partition_set.solid_selection
-        else None,
+        solids_to_execute=solids_to_execute,
         run_config=partition_data.run_config,
         mode=external_partition_set.mode,
         step_keys_to_execute=step_keys_to_execute,
@@ -265,6 +279,7 @@ def create_backfill_run(
         status=PipelineRunStatus.NOT_STARTED,
         external_pipeline_origin=external_pipeline.get_external_origin(),
         pipeline_code_origin=external_pipeline.get_python_origin(),
+        solid_selection=solid_selection,
     )
 
 
