@@ -9,7 +9,7 @@ from dagster.core.errors import (
 )
 from dagster.core.events import DagsterEvent
 from dagster.core.execution.context.system import PlanOrchestrationContext
-from dagster.core.execution.plan.state import KnownExecutionState
+from dagster.core.execution.plan.state import KnownExecutionState, StepOutputVersionData
 from dagster.core.execution.retries import RetryMode, RetryState
 from dagster.core.storage.tags import PRIORITY_TAG
 from dagster.utils.interrupts import pop_captured_interrupt
@@ -32,6 +32,7 @@ class ActiveExecution:
         retry_mode: RetryMode,
         retry_state: RetryState,
         sort_key_fn: Optional[Callable[[ExecutionStep], float]] = None,
+        step_output_versions: Optional[List[StepOutputVersionData]] = None,
     ):
         self._plan: ExecutionPlan = check.inst_param(
             execution_plan, "execution_plan", ExecutionPlan
@@ -47,6 +48,10 @@ class ActiveExecution:
             or _default_sort_key
         )
 
+        self._step_output_versions = check.opt_list_param(
+            step_output_versions, "step_output_versions", of_type=StepOutputVersionData
+        )
+
         self._context_guard: bool = False  # Prevent accidental direct use
 
         # We decide what steps to skip based on what outputs are yielded by upstream steps
@@ -119,11 +124,11 @@ class ActiveExecution:
             )
             if self._interrupted:
                 raise DagsterExecutionInterruptedError(
-                    f"Execution of pipeline was interrupted before completing the execution plan. {state_str}"
+                    f"Execution was interrupted before completing the execution plan. {state_str}"
                 )
             else:
                 raise DagsterInvariantViolationError(
-                    f"Execution of pipeline finished without completing the execution plan. {state_str}"
+                    f"Execution finished without completing the execution plan. {state_str}"
                 )
 
         # See verify_complete - steps for which we did not observe a failure/success event are in an unknown
@@ -131,7 +136,7 @@ class ActiveExecution:
         if len(self._unknown_state) > 0:
             if self._interrupted:
                 raise DagsterExecutionInterruptedError(
-                    "Execution of pipeline exited with steps {step_list} in an unknown state after "
+                    "Execution exited with steps {step_list} in an unknown state after "
                     "being interrupted.".format(step_list=self._unknown_state)
                 )
             else:
@@ -402,7 +407,10 @@ class ActiveExecution:
         elif dagster_event.is_step_success:
             self.mark_success(step_key)
         elif dagster_event.is_step_skipped:
-            self.mark_skipped(step_key)
+            # Skip events are generated by this class. They should not be sent via handle_event
+            raise DagsterInvariantViolationError(
+                "Step {} was reported as skipped from outside the ActiveExecution.".format(step_key)
+            )
         elif dagster_event.is_step_up_for_retry:
             self.mark_up_for_retry(
                 step_key,
@@ -459,6 +467,7 @@ class ActiveExecution:
         return KnownExecutionState(
             previous_retry_attempts=self._retry_state.snapshot_attempts(),
             dynamic_mappings=dict(self._successful_dynamic_outputs),
+            step_output_versions=self._step_output_versions,
         )
 
     def _prep_for_dynamic_outputs(self, step: ExecutionStep):
@@ -471,3 +480,20 @@ class ActiveExecution:
         if step_key in self._gathering_dynamic_outputs:
             self._successful_dynamic_outputs[step_key] = self._gathering_dynamic_outputs[step_key]
             self._new_dynamic_mappings = True
+
+    def rebuild_from_events(self, dagster_events: List[DagsterEvent]) -> List[ExecutionStep]:
+        """
+        Replay events to rebuild the execution state and continue after a failure.
+
+        Returns a list of steps that are possibly in flight. Current status of the event log implies
+        that the previous run worker might have crashed before launching these steps, or it may have
+        launched them but they have yet to report a STEP_START event.
+        """
+
+        self.get_steps_to_execute()
+
+        for event in dagster_events:
+            self.handle_event(event)
+            self.get_steps_to_execute()
+
+        return [self.get_step_by_key(step_key) for step_key in self._in_flight]
