@@ -5,6 +5,7 @@ from typing import Any, AsyncGenerator, Callable, Dict, Iterator, List, Set, Uni
 from dagster import check
 from dagster.core.definitions import (
     AssetMaterialization,
+    AssetObservation,
     DynamicOutput,
     ExpectationResult,
     Materialization,
@@ -22,7 +23,12 @@ from .outputs import StepOutput, StepOutputProperties
 from .utils import solid_execution_error_boundary
 
 SolidOutputUnion = Union[
-    DynamicOutput, Output, AssetMaterialization, Materialization, ExpectationResult
+    DynamicOutput,
+    Output,
+    AssetMaterialization,
+    Materialization,
+    ExpectationResult,
+    AssetObservation,
 ]
 
 
@@ -56,22 +62,33 @@ def create_step_outputs(
     ]
 
 
-def _validate_event(event: Any, solid_handle: NodeHandle) -> SolidOutputUnion:
+def _validate_event(event: Any, step_context: StepExecutionContext) -> SolidOutputUnion:
     if not isinstance(
         event,
-        (DynamicOutput, Output, AssetMaterialization, Materialization, ExpectationResult),
+        (
+            DynamicOutput,
+            Output,
+            AssetMaterialization,
+            Materialization,
+            ExpectationResult,
+            AssetObservation,
+        ),
     ):
         raise DagsterInvariantViolationError(
             (
-                "Compute function for solid {solid_name} yielded a value of type {type_} "
+                "Compute function for {described_node} yielded a value of type {type_} "
                 "rather than an instance of Output, AssetMaterialization, or ExpectationResult."
-                " Values yielded by solids must be wrapped in one of these types. If your "
-                "solid has a single output and yields no other events, you may want to use "
-                "`return` instead of `yield` in the body of your solid compute function. If "
+                " Values yielded by {node_type}s must be wrapped in one of these types. If your "
+                "{node_type} has a single output and yields no other events, you may want to use "
+                "`return` instead of `yield` in the body of your {node_type} compute function. If "
                 "you are already using `return`, and you expected to return a value of type "
                 "{type_}, you may be inadvertently returning a generator rather than the value "
                 "you expected."
-            ).format(solid_name=str(solid_handle), type_=type(event))
+            ).format(
+                described_node=step_context.describe_op(),
+                type_=type(event),
+                node_type=step_context.solid_def.node_type_str,
+            )
         )
 
     return event
@@ -96,10 +113,13 @@ def _yield_compute_results(
     if isinstance(user_event_generator, Output):
         raise DagsterInvariantViolationError(
             (
-                "Compute function for solid {solid_name} returned a Output rather than "
-                "yielding it. The compute_fn of the core SolidDefinition must yield "
+                "Compute function for {described_node} returned an Output rather than "
+                "yielding it. The compute_fn of the {node_type} must yield "
                 "its results"
-            ).format(solid_name=str(step_context.step.solid_handle))
+            ).format(
+                described_node=step_context.describe_op(),
+                node_type=step_context.solid_def.node_type_str,
+            )
         )
 
     if user_event_generator is None:
@@ -116,12 +136,12 @@ def _yield_compute_results(
             msg_fn=lambda: f"Error occurred while executing {op_label}:",
             step_context=step_context,
             step_key=step_context.step.key,
-            solid_def_name=step_context.solid_def.name,
-            solid_name=step_context.solid.name,
+            op_def_name=step_context.solid_def.name,
+            op_name=step_context.solid.name,
         ),
         user_event_generator,
     ):
-        yield _validate_event(event, step_context.step.solid_handle)
+        yield _validate_event(event, step_context)
 
 
 def execute_core_compute(
@@ -136,13 +156,12 @@ def execute_core_compute(
 
     step = step_context.step
 
-    all_results = []
+    emitted_result_names = set()
     for step_output in _yield_compute_results(step_context, inputs, compute_fn):
         yield step_output
         if isinstance(step_output, (DynamicOutput, Output)):
-            all_results.append(step_output)
+            emitted_result_names.add(step_output.output_name)
 
-    emitted_result_names = {r.output_name for r in all_results}
     solid_output_names = {output.name for output in step.step_outputs}
     omitted_outputs = solid_output_names.difference(emitted_result_names)
     if omitted_outputs:
