@@ -1,5 +1,7 @@
+import logging
 import sys
-from typing import Iterator, List
+from contextlib import ExitStack
+from typing import Iterator, List, cast
 
 from dagster import check
 from dagster.core.definitions import Failure, HookExecutionResult, RetryRequested
@@ -11,7 +13,7 @@ from dagster.core.errors import (
     user_code_error_boundary,
 )
 from dagster.core.events import DagsterEvent
-from dagster.core.execution.context.system import SystemExecutionContext, SystemStepExecutionContext
+from dagster.core.execution.context.system import PlanExecutionContext, StepExecutionContext
 from dagster.core.execution.plan.execute_step import core_dagster_event_sequence_for_step
 from dagster.core.execution.plan.objects import (
     ErrorSource,
@@ -21,14 +23,13 @@ from dagster.core.execution.plan.objects import (
     step_failure_event_from_exc_info,
 )
 from dagster.core.execution.plan.plan import ExecutionPlan
-from dagster.core.execution.retries import RetryState
 from dagster.utils.error import SerializableErrorInfo, serializable_error_info_from_exc_info
 
 
 def inner_plan_execution_iterator(
-    pipeline_context: SystemExecutionContext, execution_plan: ExecutionPlan
+    pipeline_context: PlanExecutionContext, execution_plan: ExecutionPlan
 ) -> Iterator[DagsterEvent]:
-    check.inst_param(pipeline_context, "pipeline_context", SystemExecutionContext)
+    check.inst_param(pipeline_context, "pipeline_context", PlanExecutionContext)
     check.inst_param(execution_plan, "execution_plan", ExecutionPlan)
 
     with execution_plan.start(retry_mode=pipeline_context.retry_mode) as active_execution:
@@ -38,7 +39,12 @@ def inner_plan_execution_iterator(
         # https://github.com/dagster-io/dagster/issues/811
         while not active_execution.is_complete:
             step = active_execution.get_next_step()
-            step_context = pipeline_context.for_step(step)
+            step_context = cast(
+                StepExecutionContext,
+                pipeline_context.for_step(
+                    step, active_execution.retry_state.get_attempt_count(step.key)
+                ),
+            )
             step_event_list = []
 
             missing_resources = [
@@ -55,13 +61,29 @@ def inner_plan_execution_iterator(
             )
 
             # capture all of the logs for this step
-            with pipeline_context.instance.compute_log_manager.watch(
-                step_context.pipeline_run, step_context.step.key
-            ):
+            with ExitStack() as stack:
+                log_capture_error = None
+                try:
+                    stack.enter_context(
+                        pipeline_context.instance.compute_log_manager.watch(
+                            step_context.pipeline_run, step_context.step.key
+                        )
+                    )
+                except Exception as e:
+                    log_capture_error = e
+                    logging.exception(
+                        "Exception while setting up compute log capture for step %s in run %s: %s",
+                        step_context.step.key,
+                        step_context.pipeline_run.run_id,
+                        e,
+                    )
 
-                for step_event in check.generator(
-                    _dagster_event_sequence_for_step(step_context, active_execution.retry_state)
-                ):
+                if not log_capture_error:
+                    yield DagsterEvent.capture_logs(
+                        step_context, log_key=step_context.step.key, steps=[step_context.step]
+                    )
+
+                for step_event in check.generator(_dagster_event_sequence_for_step(step_context)):
                     check.inst(step_event, DagsterEvent)
                     step_event_list.append(step_event)
                     yield step_event
@@ -69,6 +91,16 @@ def inner_plan_execution_iterator(
 
                 active_execution.verify_complete(pipeline_context, step.key)
 
+                try:
+                    stack.close()
+                except Exception as e:
+                    logging.exception(
+                        "Exception while cleaning up compute log capture for step %s in run %s: %s",
+                        step_context.step.key,
+                        step_context.pipeline_run.run_id,
+                        e,
+                    )
+
             # process skips from failures or uncovered inputs
             for event in active_execution.plan_events_iterator(pipeline_context):
                 step_event_list.append(event)
@@ -80,7 +112,7 @@ def inner_plan_execution_iterator(
 
 
 def _trigger_hook(
-    step_context: SystemStepExecutionContext, step_event_list: List[DagsterEvent]
+    step_context: StepExecutionContext, step_event_list: List[DagsterEvent]
 ) -> Iterator[DagsterEvent]:
     """Trigger hooks and record hook's operatonal events"""
     hook_defs = step_context.pipeline_def.get_all_hooks_for_handle(step_context.solid_handle)
@@ -88,6 +120,8 @@ def _trigger_hook(
     if hook_defs is None:
         return
 
+    op_label = step_context.describe_op()
+
     # when there are multiple hooks set on a solid, the hooks will run sequentially for the solid.
     # * we will not able to execute hooks asynchronously until we drop python 2.
     for hook_def in hook_defs:
@@ -96,15 +130,14 @@ def _trigger_hook(
         try:
             with user_code_error_boundary(
                 HookExecutionError,
-                lambda: "Error occurred during the execution of hook_fn triggered for solid "
-                '"{solid_name}"'.format(solid_name=step_context.solid.name),
+                lambda: f"Error occurred during the execution of hook_fn triggered for {op_label}",
+                log_manager=hook_context.log,
             ):
                 hook_execution_result = hook_def.hook_fn(hook_context, step_event_list)
 
         except HookExecutionError as hook_execution_error:
             # catch hook execution error and field a failure event instead of failing the pipeline run
-            # is_hook_completed = False
-            yield DagsterEvent.hook_errored(hook_context, hook_execution_error)
+            yield DagsterEvent.hook_errored(step_context, hook_execution_error)
             continue
 
         check.invariant(
@@ -122,15 +155,13 @@ def _trigger_hook(
             # when the triggering condition didn't meet in the hook_fn, for instance,
             # a @success_hook decorated user-defined function won't run on a failed solid
             # but internally the hook_fn still runs, so we yield HOOK_SKIPPED event instead
-            yield DagsterEvent.hook_skipped(hook_context, hook_def)
+            yield DagsterEvent.hook_skipped(step_context, hook_def)
         else:
             # hook_fn finishes successfully
-            yield DagsterEvent.hook_completed(hook_context, hook_def)
+            yield DagsterEvent.hook_completed(step_context, hook_def)
 
 
-def _dagster_event_sequence_for_step(
-    step_context: SystemStepExecutionContext, retry_state: RetryState
-) -> Iterator[DagsterEvent]:
+def _dagster_event_sequence_for_step(step_context: StepExecutionContext) -> Iterator[DagsterEvent]:
     """
     Yield a sequence of dagster events for the given step with the step context.
 
@@ -177,15 +208,16 @@ def _dagster_event_sequence_for_step(
     signaled to the user within that tool.
     """
 
-    check.inst_param(step_context, "step_context", SystemStepExecutionContext)
-    check.inst_param(retry_state, "retry_state", RetryState)
+    check.inst_param(step_context, "step_context", StepExecutionContext)
 
     try:
-        prior_attempt_count = retry_state.get_attempt_count(step_context.step.key)
         if step_context.step_launcher:
-            step_events = step_context.step_launcher.launch_step(step_context, prior_attempt_count)
+            # info all on step_context - should deprecate second arg
+            step_events = step_context.step_launcher.launch_step(
+                step_context, step_context.previous_attempt_count
+            )
         else:
-            step_events = core_dagster_event_sequence_for_step(step_context, prior_attempt_count)
+            step_events = core_dagster_event_sequence_for_step(step_context)
 
         for step_event in check.generator(step_events):
             yield step_event
@@ -201,12 +233,13 @@ def _dagster_event_sequence_for_step(
                 cls_name=retry_err_info.cls_name,
                 cause=retry_err_info.cause,
             )
+            step_context.capture_step_exception(retry_request)
             yield DagsterEvent.step_failure_event(
                 step_context=step_context,
                 step_failure_data=StepFailureData(error=fail_err, user_failure_data=None),
             )
         else:  # retries.enabled or retries.deferred
-            prev_attempts = retry_state.get_attempt_count(step_context.step.key)
+            prev_attempts = step_context.previous_attempt_count
             if prev_attempts >= retry_request.max_retries:
                 fail_err = SerializableErrorInfo(
                     message="Exceeded max_retries of {}".format(retry_request.max_retries),
@@ -214,6 +247,7 @@ def _dagster_event_sequence_for_step(
                     cls_name=retry_err_info.cls_name,
                     cause=retry_err_info.cause,
                 )
+                step_context.capture_step_exception(retry_request)
                 yield DagsterEvent.step_failure_event(
                     step_context=step_context,
                     step_failure_data=StepFailureData(error=fail_err, user_failure_data=None),
@@ -229,6 +263,7 @@ def _dagster_event_sequence_for_step(
 
     # case (2) in top comment
     except Failure as failure:
+        step_context.capture_step_exception(failure)
         yield step_failure_event_from_exc_info(
             step_context,
             sys.exc_info(),
@@ -243,6 +278,7 @@ def _dagster_event_sequence_for_step(
 
     # case (3) in top comment
     except DagsterUserCodeExecutionError as dagster_user_error:
+        step_context.capture_step_exception(dagster_user_error.user_exception)
         yield step_failure_event_from_exc_info(
             step_context,
             sys.exc_info(),
@@ -254,6 +290,7 @@ def _dagster_event_sequence_for_step(
 
     # case (4) in top comment
     except (KeyboardInterrupt, DagsterExecutionInterruptedError) as interrupt_error:
+        step_context.capture_step_exception(interrupt_error)
         yield step_failure_event_from_exc_info(
             step_context,
             sys.exc_info(),
@@ -263,6 +300,7 @@ def _dagster_event_sequence_for_step(
 
     # case (5) in top comment
     except DagsterError as dagster_error:
+        step_context.capture_step_exception(dagster_error)
         yield step_failure_event_from_exc_info(
             step_context,
             sys.exc_info(),
@@ -273,7 +311,8 @@ def _dagster_event_sequence_for_step(
             raise dagster_error
 
     # case (6) in top comment
-    except Exception as unexpected_exception:  # pylint: disable=broad-except
+    except BaseException as unexpected_exception:
+        step_context.capture_step_exception(unexpected_exception)
         yield step_failure_event_from_exc_info(
             step_context,
             sys.exc_info(),
