@@ -6,6 +6,7 @@ from dagster import check
 from dagster.core.definitions import (
     AssetKey,
     AssetMaterialization,
+    AssetObservation,
     ExpectationResult,
     Materialization,
     Output,
@@ -331,6 +332,8 @@ def core_dagster_event_sequence_for_step(
             # to do something with these in the above path eventually
             elif isinstance(user_event, (AssetMaterialization, Materialization)):
                 yield DagsterEvent.asset_materialization(step_context, user_event, input_lineage)
+            elif isinstance(user_event, AssetObservation):
+                yield DagsterEvent.asset_observation(step_context, user_event)
             elif isinstance(user_event, ExpectationResult):
                 yield DagsterEvent.step_expectation_result(step_context, user_event)
             else:
@@ -365,8 +368,9 @@ def _type_check_and_store_output(
     # to be directly captured to a dictionary after they are computed.
     if step_context.output_capture is not None:
         step_context.output_capture[step_output_handle] = output.value
-    # capture output at the step level for threading th computed output values to hook context
-    step_context.step_output_capture[step_output_handle] = output.value
+    # capture output at the step level for threading the computed output values to hook context
+    if step_context.step_output_capture is not None:
+        step_context.step_output_capture[step_output_handle] = output.value
 
     version = (
         resolve_step_output_versions(
