@@ -1,6 +1,7 @@
 import re
 
 from dagster import check
+from dagster.core.definitions.version_strategy import OpVersionContext, ResourceVersionContext
 from dagster.core.errors import DagsterInvariantViolationError
 from dagster.core.execution.plan.outputs import StepOutputHandle
 from dagster.core.execution.plan.step import is_executable_step
@@ -96,48 +97,58 @@ def resolve_step_versions(pipeline_def, execution_plan, resolved_run_config):
 
         solid_name = str(step.solid_handle)
 
+        solid_config = resolved_run_config.solids[solid_name].config
+
         solid_def_version = None
         if solid_def.version is not None:
             solid_def_version = solid_def.version
         elif pipeline_def.version_strategy is not None:
-            solid_def_version = pipeline_def.version_strategy.get_solid_version(solid_def)
+            version_context = OpVersionContext(op_def=solid_def, op_config=solid_config)
+            solid_def_version = pipeline_def.version_strategy.get_op_version(version_context)
 
         if solid_def_version is None:
+            node_label = f"{solid_def.node_type_str} '{solid_def.name}'"
             raise DagsterInvariantViolationError(
-                f"While using memoization, version for solid '{solid_def.name}' was None. Please "
+                f"While using memoization, version for {node_label} was None. Please "
                 "either provide a versioning strategy for your job, or provide a version using the "
                 "solid decorator."
             )
 
         check_valid_version(solid_def_version)
 
-        solid_config_version = resolve_config_version(resolved_run_config.solids[solid_name].config)
+        solid_config_version = resolve_config_version(solid_config)
 
         resource_versions_for_solid = []
         for resource_key in solid_def.required_resource_keys:
             if resource_key not in resource_versions:
+
+                resource_config = resolved_run_config.resources[resource_key].config
+                resource_config_version = resolve_config_version(resource_config)
+
                 resource_def = resource_defs[resource_key]
                 resource_def_version = None
                 if resource_def.version is not None:
                     resource_def_version = resource_def.version
                 else:
-                    resource_def_version = pipeline_def.version_strategy.get_resource_version(
-                        resource_def
+                    version_context = ResourceVersionContext(
+                        resource_def=resource_def, resource_config=resource_config
                     )
-                if resource_def_version is None:
-                    raise DagsterInvariantViolationError(
-                        f"While using memoization, version for resource '{resource_key}' was None. Please "
-                        "either provide a versioning strategy for your job, or provide a version using the "
-                        "resource decorator."
+                    resource_def_version = pipeline_def.version_strategy.get_resource_version(
+                        version_context
                     )
 
-                check_valid_version(resource_def_version)
-                resource_config = resolved_run_config.resources[resource_key].config
-                resource_config_version = resolve_config_version(resource_config)
-                resource_versions[resource_key] = join_and_hash(
-                    resource_config_version, resource_def_version
-                )
-            resource_versions_for_solid.append(resource_versions[resource_key])
+                if resource_def_version is not None:
+
+                    check_valid_version(resource_def_version)
+                    resource_versions[resource_key] = join_and_hash(
+                        resource_config_version, resource_def_version
+                    )
+                else:
+
+                    resource_versions[resource_key] = join_and_hash(resource_config)
+
+            if resource_versions[resource_key] is not None:
+                resource_versions_for_solid.append(resource_versions[resource_key])
         solid_resources_version = join_and_hash(*resource_versions_for_solid)
         solid_version = join_and_hash(
             solid_def_version, solid_config_version, solid_resources_version
