@@ -4,9 +4,8 @@ from typing import Optional, Sequence
 
 from dagster import check
 from dagster.core.definitions.events import AssetKey
-from dagster.core.definitions.run_request import JobType
+from dagster.core.definitions.run_request import InstigatorType
 from dagster.core.definitions.sensor_definition import DEFAULT_SENSOR_DAEMON_INTERVAL
-from dagster.core.errors import DagsterInvariantViolationError
 from dagster.core.execution.plan.handle import ResolvedFromDynamicStepHandle, StepHandle
 from dagster.core.origin import PipelinePythonOrigin
 from dagster.core.snap import ExecutionPlanSnapshot
@@ -352,25 +351,13 @@ class ExternalExecutionPlan:
     was compiled in another process or persisted in an instance.
     """
 
-    def __init__(self, execution_plan_snapshot, represented_pipeline):
+    def __init__(self, execution_plan_snapshot):
         self.execution_plan_snapshot = check.inst_param(
             execution_plan_snapshot, "execution_plan_snapshot", ExecutionPlanSnapshot
         )
-        self.represented_pipeline = check.inst_param(
-            represented_pipeline, "represented_pipeline", RepresentedPipeline
-        )
 
         self._step_index = {step.key: step for step in self.execution_plan_snapshot.steps}
 
-        if (
-            execution_plan_snapshot.pipeline_snapshot_id
-            != represented_pipeline.identifying_pipeline_snapshot_id
-        ):
-            raise DagsterInvariantViolationError(
-                "The target snapshot ID from the execution plan snapshot does not match the "
-                "passed in target snapshot. "
-            )
-
         self._step_keys_in_plan = (
             set(execution_plan_snapshot.step_keys_to_execute)
             if execution_plan_snapshot.step_keys_to_execute
@@ -502,16 +489,18 @@ class ExternalSchedule:
     # ScheduleState that represents the state of the schedule
     # when there is no row in the schedule DB (for example, when
     # the schedule is first created in code)
-    def get_default_instigation_state(self, instance):
-        from dagster.core.scheduler.job import JobState, JobStatus, ScheduleJobData
+    def get_default_instigation_state(self):
+        from dagster.core.scheduler.instigation import (
+            InstigatorState,
+            InstigatorStatus,
+            ScheduleInstigatorData,
+        )
 
-        return JobState(
+        return InstigatorState(
             self.get_external_origin(),
-            JobType.SCHEDULE,
-            JobStatus.STOPPED,
-            ScheduleJobData(
-                self.cron_schedule, start_timestamp=None, scheduler=instance.scheduler_class
-            ),
+            InstigatorType.SCHEDULE,
+            InstigatorStatus.STOPPED,
+            ScheduleInstigatorData(self.cron_schedule, start_timestamp=None),
         )
 
     def execution_time_iterator(self, start_timestamp):
@@ -582,14 +571,18 @@ class ExternalSensor:
     def get_external_origin_id(self):
         return self.get_external_origin().get_id()
 
-    def get_default_instigation_state(self, _instance):
-        from dagster.core.scheduler.job import JobState, JobStatus, SensorJobData
+    def get_default_instigation_state(self):
+        from dagster.core.scheduler.instigation import (
+            InstigatorState,
+            InstigatorStatus,
+            SensorInstigatorData,
+        )
 
-        return JobState(
+        return InstigatorState(
             self.get_external_origin(),
-            JobType.SENSOR,
-            JobStatus.STOPPED,
-            SensorJobData(min_interval=self.min_interval_seconds),
+            InstigatorType.SENSOR,
+            InstigatorStatus.STOPPED,
+            SensorInstigatorData(min_interval=self.min_interval_seconds),
         )
 
     @property
