@@ -1,9 +1,11 @@
 import logging
 import time
+from typing import Any, Dict, List, NamedTuple, Optional
 
 from dagster import DagsterEvent, DagsterEventType, IntSource, String, check
+from dagster.builtins import Bool
 from dagster.config import Field
-from dagster.config.config_type import Array, Noneable
+from dagster.config.config_type import Array, Noneable, ScalarUnion
 from dagster.config.field_utils import Shape
 from dagster.core.events.log import EventLogEntry
 from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus
@@ -12,6 +14,15 @@ from dagster.serdes import ConfigurableClass, ConfigurableClassData
 from .base import RunCoordinator, SubmitRunContext
 
 
+class RunQueueConfig(
+    NamedTuple(
+        "_RunQueueConfig",
+        [("max_concurrent_runs", int), ("tag_concurrency_limits", Optional[List[Dict[str, Any]]])],
+    )
+):
+    pass
+
+
 class QueuedRunCoordinator(RunCoordinator, ConfigurableClass):
     """
     Sends runs to the dequeuer process via the run storage. Requires the external process to be
@@ -26,14 +37,14 @@ class QueuedRunCoordinator(RunCoordinator, ConfigurableClass):
         inst_data=None,
     ):
         self._inst_data = check.opt_inst_param(inst_data, "inst_data", ConfigurableClassData)
-        self.max_concurrent_runs = check.opt_int_param(
+        self._max_concurrent_runs = check.opt_int_param(
             max_concurrent_runs, "max_concurrent_runs", 10
         )
-        self.tag_concurrency_limits = check.opt_list_param(
+        self._tag_concurrency_limits = check.opt_list_param(
             tag_concurrency_limits,
             "tag_concurrency_limits",
         )
-        self.dequeue_interval_seconds = check.opt_int_param(
+        self._dequeue_interval_seconds = check.opt_int_param(
             dequeue_interval_seconds, "dequeue_interval_seconds", 5
         )
 
@@ -43,6 +54,16 @@ class QueuedRunCoordinator(RunCoordinator, ConfigurableClass):
     def inst_data(self):
         return self._inst_data
 
+    def get_run_queue_config(self):
+        return RunQueueConfig(
+            max_concurrent_runs=self._max_concurrent_runs,
+            tag_concurrency_limits=self._tag_concurrency_limits,
+        )
+
+    @property
+    def dequeue_interval_seconds(self):
+        return self._dequeue_interval_seconds
+
     @classmethod
     def config_type(cls):
         return {
@@ -53,7 +74,13 @@ class QueuedRunCoordinator(RunCoordinator, ConfigurableClass):
                         Shape(
                             {
                                 "key": String,
-                                "value": Field(String, is_required=False),
+                                "value": Field(
+                                    ScalarUnion(
+                                        scalar_type=String,
+                                        non_scalar_schema=Shape({"applyLimitPerUniqueValue": Bool}),
+                                    ),
+                                    is_required=False,
+                                ),
                                 "limit": Field(int),
                             }
                         )
