@@ -2,7 +2,7 @@ import warnings
 from collections import namedtuple
 from datetime import datetime
 from enum import Enum
-from typing import Any, Dict, NamedTuple, Type
+from typing import Any, Dict, NamedTuple, Optional, Type
 
 from dagster import check
 from dagster.core.origin import PipelinePythonOrigin
@@ -434,7 +434,8 @@ register_serdes_tuple_fallbacks({"PipelineRun": DagsterRun})
 @whitelist_for_serdes
 class PipelineRunsFilter(
     namedtuple(
-        "_PipelineRunsFilter", "run_ids pipeline_name statuses tags snapshot_id updated_after mode"
+        "_PipelineRunsFilter",
+        "run_ids pipeline_name statuses tags snapshot_id updated_after mode created_before",
     )
 ):
     def __new__(
@@ -446,6 +447,7 @@ class PipelineRunsFilter(
         snapshot_id=None,
         updated_after=None,
         mode=None,
+        created_before=None,
     ):
         return super(PipelineRunsFilter, cls).__new__(
             cls,
@@ -456,6 +458,7 @@ class PipelineRunsFilter(
             snapshot_id=check.opt_str_param(snapshot_id, "snapshot_id"),
             updated_after=check.opt_inst_param(updated_after, "updated_after", datetime),
             mode=check.opt_str_param(mode, "mode"),
+            created_before=check.opt_inst_param(created_before, "created_before", datetime),
         )
 
     @staticmethod
@@ -475,15 +478,42 @@ class PipelineRunsFilter(
         return PipelineRunsFilter(tags=PipelineRun.tags_for_backfill_id(backfill_id))
 
 
-class RunRecord(NamedTuple):
+class RunRecord(
+    NamedTuple(
+        "_RunRecord",
+        [
+            ("storage_id", int),
+            ("pipeline_run", PipelineRun),
+            ("create_timestamp", datetime),
+            ("update_timestamp", datetime),
+            ("start_time", Optional[float]),
+            ("end_time", Optional[float]),
+        ],
+    )
+):
     """Internal representation of a run record, as stored in a
     :py:class:`~dagster.core.storage.runs.RunStorage`.
     """
 
-    storage_id: int
-    pipeline_run: PipelineRun
-    create_timestamp: datetime
-    update_timestamp: datetime
+    def __new__(
+        cls,
+        storage_id,
+        pipeline_run,
+        create_timestamp,
+        update_timestamp,
+        start_time=None,
+        end_time=None,
+    ):
+        return super(RunRecord, cls).__new__(
+            cls,
+            storage_id=check.int_param(storage_id, "storage_id"),
+            pipeline_run=check.inst_param(pipeline_run, "pipeline_run", PipelineRun),
+            create_timestamp=check.inst_param(create_timestamp, "create_timestamp", datetime),
+            update_timestamp=check.inst_param(update_timestamp, "update_timestamp", datetime),
+            # start_time and end_time fields will be populated once the run has started and ended, respectively, but will be None beforehand.
+            start_time=check.opt_float_param(start_time, "start_time"),
+            end_time=check.opt_float_param(end_time, "end_time"),
+        )
 
 
 ###################################################################################################
