@@ -1,5 +1,5 @@
 from dagster.api.snapshot_schedule import sync_get_external_schedule_execution_data_ephemeral_grpc
-from dagster.core.definitions.schedule import ScheduleExecutionData
+from dagster.core.definitions.schedule_definition import ScheduleExecutionData
 from dagster.core.test_utils import instance_for_test
 from dagster.seven import get_current_datetime_in_utc
 
