@@ -1,9 +1,9 @@
 import pytest
 from click.testing import CliRunner
-
 from dagster import AssetKey, AssetMaterialization, Output, execute_pipeline, pipeline, solid
 from dagster.cli.asset import asset_wipe_command
 from dagster.core.instance import DagsterInstance
+from dagster.seven import json
 
 
 @pytest.fixture(name="asset_instance")
@@ -11,7 +11,8 @@ def mock_asset_instance(mocker):
     # can use the ephemeral instance, since the default InMemoryEventLogStorage is asset aware
     instance = DagsterInstance.ephemeral()
     mocker.patch(
-        "dagster.core.instance.DagsterInstance.get", return_value=instance,
+        "dagster.core.instance.DagsterInstance.get",
+        return_value=instance,
     )
     yield instance
 
@@ -47,7 +48,7 @@ def pipeline_two():
     solid_two()
 
 
-def test_asset_wipe_errors():
+def test_asset_wipe_errors(asset_instance):  # pylint: disable=unused-argument
     runner = CliRunner()
     result = runner.invoke(asset_wipe_command)
     assert result.exit_code == 2
@@ -56,12 +57,12 @@ def test_asset_wipe_errors():
         in result.output
     )
 
-    result = runner.invoke(asset_wipe_command, ["--all", "path.to.asset_key"])
+    result = runner.invoke(asset_wipe_command, ["--all", json.dumps(["path", "to", "asset_key"])])
     assert result.exit_code == 2
     assert "Error, cannot use more than one of: asset key, `--all`." in result.output
 
 
-def test_asset_exit():
+def test_asset_exit(asset_instance):  # pylint: disable=unused-argument
     runner = CliRunner()
     result = runner.invoke(asset_wipe_command, ["--all"], input="NOT_DELETE\n")
     assert result.exit_code == 0
@@ -72,15 +73,18 @@ def test_asset_single_wipe(asset_instance):
     runner = CliRunner()
     execute_pipeline(pipeline_one, instance=asset_instance)
     execute_pipeline(pipeline_two, instance=asset_instance)
-    assert asset_instance.is_asset_aware
     asset_keys = asset_instance.all_asset_keys()
     assert len(asset_keys) == 4
 
-    result = runner.invoke(asset_wipe_command, ["path.to.asset_3"], input="DELETE\n")
+    result = runner.invoke(
+        asset_wipe_command, [json.dumps(["path", "to", "asset_3"])], input="DELETE\n"
+    )
     assert result.exit_code == 0
     assert "Removed asset indexes from event logs" in result.output
 
-    result = runner.invoke(asset_wipe_command, ["path.to.asset_4"], input="DELETE\n")
+    result = runner.invoke(
+        asset_wipe_command, [json.dumps(["path", "to", "asset_4"])], input="DELETE\n"
+    )
     assert result.exit_code == 0
     assert "Removed asset indexes from event logs" in result.output
 
@@ -92,11 +96,14 @@ def test_asset_multi_wipe(asset_instance):
     runner = CliRunner()
     execute_pipeline(pipeline_one, instance=asset_instance)
     execute_pipeline(pipeline_two, instance=asset_instance)
-    assert asset_instance.is_asset_aware
     asset_keys = asset_instance.all_asset_keys()
     assert len(asset_keys) == 4
 
-    result = runner.invoke(asset_wipe_command, ["path.to.asset_3", "asset_1"], input="DELETE\n")
+    result = runner.invoke(
+        asset_wipe_command,
+        [json.dumps(["path", "to", "asset_3"]), json.dumps(["asset_1"])],
+        input="DELETE\n",
+    )
     assert result.exit_code == 0
     assert "Removed asset indexes from event logs" in result.output
     asset_keys = asset_instance.all_asset_keys()
@@ -107,7 +114,6 @@ def test_asset_wipe_all(asset_instance):
     runner = CliRunner()
     execute_pipeline(pipeline_one, instance=asset_instance)
     execute_pipeline(pipeline_two, instance=asset_instance)
-    assert asset_instance.is_asset_aware
     asset_keys = asset_instance.all_asset_keys()
     assert len(asset_keys) == 4
 
