@@ -4,9 +4,13 @@ import pytest
 from dagster import (
     DagsterEventType,
     ModeDefinition,
+    NodeInvocation,
     PipelineDefinition,
-    SolidInvocation,
+    build_hook_context,
+    check,
     execute_pipeline,
+    graph,
+    op,
     pipeline,
     reconstructable,
     resource,
@@ -50,7 +54,7 @@ def test_hook():
         solid_defs=[a_solid],
         name="test",
         dependencies={
-            SolidInvocation("a_solid", "a_solid_with_hook", hook_defs={a_hook, named_hook}): {}
+            NodeInvocation("a_solid", "a_solid_with_hook", hook_defs={a_hook, named_hook}): {}
         },
     )
 
@@ -76,7 +80,7 @@ def test_hook_user_error():
     a_pipeline = PipelineDefinition(
         solid_defs=[a_solid],
         name="test",
-        dependencies={SolidInvocation("a_solid", "a_solid_with_hook", hook_defs={error_hook}): {}},
+        dependencies={NodeInvocation("a_solid", "a_solid_with_hook", hook_defs={error_hook}): {}},
     )
 
     result = execute_pipeline(a_pipeline)
@@ -125,7 +129,7 @@ def test_hook_with_resource():
     a_pipeline = PipelineDefinition(
         solid_defs=[a_solid],
         name="test",
-        dependencies={SolidInvocation("a_solid", "a_solid_with_hook", hook_defs={a_hook}): {}},
+        dependencies={NodeInvocation("a_solid", "a_solid_with_hook", hook_defs={a_hook}): {}},
         mode_defs=[ModeDefinition(resource_defs={"resource_a": resource_a})],
     )
 
@@ -145,12 +149,12 @@ def test_hook_resource_error():
 
     with pytest.raises(
         DagsterInvalidDefinitionError,
-        match='Resource key "resource_b" is required by hook "a_hook"',
+        match="resource key 'resource_b' is required by hook 'a_hook'",
     ):
         PipelineDefinition(
             solid_defs=[a_solid],
             name="test",
-            dependencies={SolidInvocation("a_solid", "a_solid_with_hook", hook_defs={a_hook}): {}},
+            dependencies={NodeInvocation("a_solid", "a_solid_with_hook", hook_defs={a_hook}): {}},
             mode_defs=[ModeDefinition(resource_defs={"resource_a": resource_a})],
         )
 
@@ -185,12 +189,12 @@ def test_success_hook():
         solid_defs=[succeeded_solid, failed_solid],
         name="test",
         dependencies={
-            SolidInvocation(
+            NodeInvocation(
                 "succeeded_solid",
                 "succeeded_solid_with_hook",
                 hook_defs={a_success_hook, named_success_hook, success_hook_resource},
             ): {},
-            SolidInvocation(
+            NodeInvocation(
                 "failed_solid",
                 "failed_solid_with_hook",
                 hook_defs={a_success_hook, named_success_hook},
@@ -240,12 +244,12 @@ def test_failure_hook():
         solid_defs=[failed_solid, succeeded_solid],
         name="test",
         dependencies={
-            SolidInvocation(
+            NodeInvocation(
                 "failed_solid",
                 "failed_solid_with_hook",
                 hook_defs={a_failure_hook, named_failure_hook, failure_hook_resource},
             ): {},
-            SolidInvocation(
+            NodeInvocation(
                 "succeeded_solid",
                 "succeeded_solid_with_hook",
                 hook_defs={a_failure_hook, named_failure_hook},
@@ -281,8 +285,8 @@ def test_success_hook_event():
         solid_defs=[a_solid, failed_solid],
         name="test",
         dependencies={
-            SolidInvocation("a_solid", hook_defs={a_hook}): {},
-            SolidInvocation("failed_solid", hook_defs={a_hook}): {},
+            NodeInvocation("a_solid", hook_defs={a_hook}): {},
+            NodeInvocation("failed_solid", hook_defs={a_hook}): {},
         },
     )
 
@@ -316,8 +320,8 @@ def test_failure_hook_event():
         solid_defs=[a_solid, failed_solid],
         name="test",
         dependencies={
-            SolidInvocation("a_solid", hook_defs={a_hook}): {},
-            SolidInvocation("failed_solid", hook_defs={a_hook}): {},
+            NodeInvocation("a_solid", hook_defs={a_hook}): {},
+            NodeInvocation("failed_solid", hook_defs={a_hook}): {},
         },
     )
 
@@ -386,3 +390,121 @@ def test_hook_decorator():
     retry_policy = a_pipeline.get_retry_policy_for_handle(NodeHandle("a_solid", parent=None))
     assert isinstance(retry_policy, RetryPolicy)
     assert retry_policy.max_retries == 3
+
+
+def test_hook_with_resource_to_resource_dep():
+    called = {}
+
+    @resource(required_resource_keys={"resource_a"})
+    def resource_b(context):
+        return context.resources.resource_a
+
+    @event_list_hook(required_resource_keys={"resource_b"})
+    def hook_requires_b(context, _):
+        called[context.solid.name] = True
+        assert context.resources.resource_b == 1
+        return HookExecutionResult(hook_name="a_hook")
+
+    @solid
+    def basic_solid():
+        pass
+
+    mode_def = ModeDefinition(resource_defs={"resource_a": resource_a, "resource_b": resource_b})
+
+    # Check that resource-to-resource dependency is caught when providing hook to solid
+    @pipeline(mode_defs=[mode_def])
+    def basic_pipeline():
+        basic_solid.with_hooks({hook_requires_b})()
+
+    result = execute_pipeline(basic_pipeline)
+    assert result.success
+    assert called.get("basic_solid")
+
+    # Check that resource-to-resource dependency is caught when providing hook to pipeline
+    @pipeline(mode_defs=[mode_def])
+    def basic_pipeline_gonna_use_hooks():
+        basic_solid()
+
+    called = {}
+    basic_hook_pipeline = basic_pipeline_gonna_use_hooks.with_hooks({hook_requires_b})
+
+    result = execute_pipeline(basic_hook_pipeline)
+    assert result.success
+    assert called.get("basic_solid")
+
+
+def test_hook_graph_job_op():
+    called = {}
+    op_output = "hook_op_output"
+
+    @success_hook(required_resource_keys={"resource_a"})
+    def hook_one(context):
+        assert context.op.name
+        called[context.hook_def.name] = called.get(context.hook_def.name, 0) + 1
+
+    @success_hook()
+    def hook_two(context):
+        assert not context.op_config
+        assert not context.op_exception
+        assert context.op_output_values["result"] == op_output
+        called[context.hook_def.name] = called.get(context.hook_def.name, 0) + 1
+
+    @op
+    def hook_op(_):
+        return op_output
+
+    ctx = build_hook_context(resources={"resource_a": resource_a}, op=hook_op)
+    hook_one(ctx)
+    assert called.get("hook_one") == 1
+
+    @graph
+    def run_success_hook():
+        hook_op.with_hooks({hook_one, hook_two})()
+
+    success_hook_job = run_success_hook.to_job(resource_defs={"resource_a": resource_a})
+    assert success_hook_job.execute_in_process().success
+
+    assert called.get("hook_one") == 2
+    assert called.get("hook_two") == 1
+
+
+def test_hook_context_op_solid_provided():
+    @op
+    def hook_op(_):
+        pass
+
+    with pytest.raises(check.CheckError):
+        build_hook_context(op=hook_op, solid=hook_op)
+
+
+def test_hook_decorator_graph_job_op():
+    called_hook_to_solids = defaultdict(list)
+
+    @success_hook
+    def a_success_hook(context):
+        called_hook_to_solids[context.hook_def.name].append(context.solid.name)
+
+    @op
+    def my_op(_):
+        pass
+
+    @graph
+    def a_graph():
+        my_op()
+
+    assert a_graph.to_job(hooks={a_success_hook}).execute_in_process().success
+    assert called_hook_to_solids["a_success_hook"][0] == "my_op"
+
+
+def test_job_hook_context_job_name():
+    my_job_name = "my_test_job_name"
+
+    @success_hook
+    def a_success_hook(context):
+        assert context.job_name == my_job_name
+
+    @graph
+    def a_graph():
+        pass
+
+    assert a_graph.to_job(name=my_job_name, hooks={a_success_hook}).execute_in_process().success
