@@ -3,6 +3,7 @@ import re
 import pytest
 import yaml
 from dagster import PipelineDefinition, check, execute_pipeline, pipeline, solid
+from dagster.check import CheckError
 from dagster.config import Field
 from dagster.core.errors import (
     DagsterHomeNotSetError,
@@ -11,6 +12,7 @@ from dagster.core.errors import (
 )
 from dagster.core.execution.api import create_execution_plan
 from dagster.core.instance import DagsterInstance, InstanceRef
+from dagster.core.launcher import LaunchRunContext, RunLauncher
 from dagster.core.run_coordinator.queued_run_coordinator import QueuedRunCoordinator
 from dagster.core.snap import (
     create_execution_plan_snapshot_id,
@@ -18,6 +20,7 @@ from dagster.core.snap import (
     snapshot_from_execution_plan,
 )
 from dagster.core.test_utils import create_run_for_test, environ, instance_for_test
+from dagster.serdes import ConfigurableClass
 from dagster.serdes.config_class import ConfigurableClassData
 from dagster_tests.api_tests.utils import get_bar_workspace
 
@@ -131,6 +134,88 @@ def test_submit_run():
             assert instance.run_coordinator.queue()[0].run_id == "foo-bar"
 
 
+def test_get_required_daemon_types():
+    from dagster.daemon.daemon import (
+        SensorDaemon,
+        BackfillDaemon,
+        SchedulerDaemon,
+        MonitoringDaemon,
+    )
+
+    with instance_for_test() as instance:
+        assert instance.get_required_daemon_types() == [
+            SensorDaemon.daemon_type(),
+            BackfillDaemon.daemon_type(),
+            SchedulerDaemon.daemon_type(),
+        ]
+
+    with instance_for_test(
+        overrides={
+            "run_launcher": {
+                "module": "dagster_tests.daemon_tests.test_monitoring_daemon",
+                "class": "TestRunLauncher",
+            },
+            "run_monitoring": {"enabled": True},
+        }
+    ) as instance:
+        assert instance.get_required_daemon_types() == [
+            SensorDaemon.daemon_type(),
+            BackfillDaemon.daemon_type(),
+            SchedulerDaemon.daemon_type(),
+            MonitoringDaemon.daemon_type(),
+        ]
+
+
+def test_run_monitoring():
+    with pytest.raises(CheckError):
+        with instance_for_test(
+            overrides={
+                "run_monitoring": {"enabled": True},
+            }
+        ):
+            pass
+
+    settings = {"enabled": True}
+    with instance_for_test(
+        overrides={
+            "run_launcher": {
+                "module": "dagster_tests.daemon_tests.test_monitoring_daemon",
+                "class": "TestRunLauncher",
+            },
+            "run_monitoring": settings,
+        }
+    ) as instance:
+        assert instance.run_monitoring_enabled
+        assert instance.run_monitoring_settings == settings
+        assert instance.run_monitoring_max_resume_run_attempts == 3
+
+    settings = {"enabled": True, "max_resume_run_attempts": 5}
+    with instance_for_test(
+        overrides={
+            "run_launcher": {
+                "module": "dagster_tests.daemon_tests.test_monitoring_daemon",
+                "class": "TestRunLauncher",
+            },
+            "run_monitoring": settings,
+        }
+    ) as instance:
+        assert instance.run_monitoring_enabled
+        assert instance.run_monitoring_settings == settings
+        assert instance.run_monitoring_max_resume_run_attempts == 5
+
+
+def test_cancellation_thread():
+    with instance_for_test(
+        overrides={
+            "run_monitoring": {"cancellation_thread_poll_interval_seconds": 300},
+        }
+    ) as instance:
+        assert instance.cancellation_thread_poll_interval_seconds == 300
+
+    with instance_for_test() as instance:
+        assert instance.cancellation_thread_poll_interval_seconds == 10
+
+
 def test_dagster_home_not_set():
     with environ({"DAGSTER_HOME": ""}):
         with pytest.raises(
@@ -269,3 +354,31 @@ def test_instance_subclass():
             }
         ) as subclass_instance:
             pass
+
+
+# class that doesn't implement needed methods on ConfigurableClass
+class InvalidRunLauncher(RunLauncher, ConfigurableClass):
+    def can_terminate(self, run_id):
+        return False
+
+    def launch_run(self, context: LaunchRunContext) -> None:
+        pass
+
+    def terminate(self, run_id):
+        pass
+
+
+def test_configurable_class_missing_methods():
+    with pytest.raises(
+        NotImplementedError,
+        match="InvalidRunLauncher must implement the config_type classmethod",
+    ):
+        with instance_for_test(
+            overrides={
+                "run_launcher": {
+                    "module": "dagster_tests.core_tests.instance_tests.test_instance",
+                    "class": "InvalidRunLauncher",
+                }
+            }
+        ):
+            pass
