@@ -3,13 +3,17 @@ from typing import Callable, List, Optional
 
 import pendulum
 import pytest
+from dagster import (
+    DailyPartitionsDefinition,
+    DynamicPartitionsDefinition,
+    HourlyPartitionsDefinition,
+    StaticPartitionsDefinition,
+)
 from dagster.check import CheckError
 from dagster.core.definitions.partition import (
-    DynamicPartitionParams,
     Partition,
+    ScheduleTimeBasedPartitionsDefinition,
     ScheduleType,
-    StaticPartitionParams,
-    TimeBasedPartitionParams,
 )
 from dagster.seven.compat.pendulum import create_pendulum_time
 from dagster.utils.partitions import DEFAULT_HOURLY_FORMAT_WITH_TIMEZONE
@@ -29,13 +33,16 @@ def assert_expected_partitions(
 
 
 @pytest.mark.parametrize(
-    argnames=["partitions"],
-    argvalues=[([Partition("a_partition")],), ([Partition(x) for x in range(10)],)],
+    argnames=["partition_keys"],
+    argvalues=[(["a_partition"],), ([str(x) for x in range(10)],)],
 )
-def test_static_partition_params(partitions: List[Partition]):
-    partition_params = StaticPartitionParams(partitions)
+def test_static_partitions(partition_keys: List[str]):
+    static_partitions = StaticPartitionsDefinition(partition_keys)
 
-    assert partition_params.get_partitions() == partitions
+    assert [(p.name, p.value) for p in static_partitions.get_partitions()] == [
+        (p, p) for p in partition_keys
+    ]
+    assert static_partitions.get_partition_keys() == partition_keys
 
 
 @pytest.mark.parametrize(
@@ -85,7 +92,7 @@ def test_static_partition_params(partitions: List[Partition]):
         ),
     ],
 )
-def test_time_based_partition_params_invariants(
+def test_time_based_partitions_invariants(
     schedule_type: ScheduleType,
     start: datetime,
     execution_day: Optional[int],
@@ -93,7 +100,7 @@ def test_time_based_partition_params_invariants(
     error_message_regex: str,
 ):
     with pytest.raises(CheckError, match=error_message_regex):
-        TimeBasedPartitionParams(
+        ScheduleTimeBasedPartitionsDefinition(
             schedule_type=schedule_type,
             start=start,
             execution_day=execution_day,
@@ -226,7 +233,7 @@ def test_time_based_partition_params_invariants(
         ),
     ],
 )
-def test_time_partition_params_daily_partitions(
+def test_time_partitions_daily_partitions(
     start: datetime,
     execution_time: time,
     end: datetime,
@@ -235,7 +242,7 @@ def test_time_partition_params_daily_partitions(
     expected_partitions: List[str],
 ):
     with pendulum.test(current_time):
-        partition_params = TimeBasedPartitionParams(
+        partitions = ScheduleTimeBasedPartitionsDefinition(
             schedule_type=ScheduleType.DAILY,
             start=start,
             execution_time=execution_time,
@@ -243,7 +250,7 @@ def test_time_partition_params_daily_partitions(
             offset=partition_days_offset,
         )
 
-        assert_expected_partitions(partition_params.get_partitions(), expected_partitions)
+        assert_expected_partitions(partitions.get_partitions(), expected_partitions)
 
 
 @pytest.mark.parametrize(
@@ -331,7 +338,7 @@ def test_time_partition_params_daily_partitions(
         ),
     ],
 )
-def test_time_partition_params_monthly_partitions(
+def test_time_partitions_monthly_partitions(
     start: datetime,
     end: datetime,
     partition_months_offset: Optional[int],
@@ -339,7 +346,7 @@ def test_time_partition_params_monthly_partitions(
     expected_partitions: List[str],
 ):
     with pendulum.test(current_time):
-        partition_params = TimeBasedPartitionParams(
+        partitions = ScheduleTimeBasedPartitionsDefinition(
             schedule_type=ScheduleType.MONTHLY,
             start=start,
             execution_time=time(1, 20),
@@ -348,7 +355,7 @@ def test_time_partition_params_monthly_partitions(
             offset=partition_months_offset,
         )
 
-        assert_expected_partitions(partition_params.get_partitions(), expected_partitions)
+        assert_expected_partitions(partitions.get_partitions(), expected_partitions)
 
 
 @pytest.mark.parametrize(
@@ -436,7 +443,7 @@ def test_time_partition_params_monthly_partitions(
         ),
     ],
 )
-def test_time_partition_params_weekly_partitions(
+def test_time_partitions_weekly_partitions(
     start: datetime,
     end: datetime,
     partition_weeks_offset: Optional[int],
@@ -444,7 +451,7 @@ def test_time_partition_params_weekly_partitions(
     expected_partitions: List[str],
 ):
     with pendulum.test(current_time):
-        partition_params = TimeBasedPartitionParams(
+        partitions = ScheduleTimeBasedPartitionsDefinition(
             schedule_type=ScheduleType.WEEKLY,
             start=start,
             execution_time=time(1, 20),
@@ -453,7 +460,7 @@ def test_time_partition_params_weekly_partitions(
             offset=partition_weeks_offset,
         )
 
-        assert_expected_partitions(partition_params.get_partitions(), expected_partitions)
+        assert_expected_partitions(partitions.get_partitions(), expected_partitions)
 
 
 @pytest.mark.parametrize(
@@ -640,7 +647,7 @@ def test_time_partition_params_weekly_partitions(
         ),
     ],
 )
-def test_time_partition_params_hourly_partitions(
+def test_time_partitions_hourly_partitions(
     start: datetime,
     end: datetime,
     timezone: Optional[str],
@@ -649,7 +656,7 @@ def test_time_partition_params_hourly_partitions(
     expected_partitions: List[str],
 ):
     with pendulum.test(current_time):
-        partition_params = TimeBasedPartitionParams(
+        partitions = ScheduleTimeBasedPartitionsDefinition(
             schedule_type=ScheduleType.HOURLY,
             start=start,
             execution_time=time(0, 1),
@@ -659,7 +666,7 @@ def test_time_partition_params_hourly_partitions(
             offset=partition_hours_offset,
         )
 
-        assert_expected_partitions(partition_params.get_partitions(), expected_partitions)
+        assert_expected_partitions(partitions.get_partitions(), expected_partitions)
 
 
 @pytest.mark.parametrize(
@@ -669,7 +676,47 @@ def test_time_partition_params_hourly_partitions(
         (lambda _current_time: [Partition(x) for x in range(10)],),
     ],
 )
-def test_dynamic_partitions(partition_fn: Callable[[Optional[datetime]], List[Partition]]):
-    partition_params = DynamicPartitionParams(partition_fn)
+def test_dynamic_partitions_partitions(
+    partition_fn: Callable[[Optional[datetime]], List[Partition]]
+):
+    partitions = DynamicPartitionsDefinition(partition_fn)
 
-    assert partition_params.get_partitions() == partition_fn(None)
+    assert [(p.name, p.value) for p in partitions.get_partitions()] == [
+        (p.name, p.value) for p in partition_fn(None)
+    ]
+
+    assert partitions.get_partition_keys() == [p.name for p in partition_fn(None)]
+
+
+@pytest.mark.parametrize(
+    argnames=["partition_fn"],
+    argvalues=[
+        (lambda _current_time: ["a_partition"],),
+        (lambda _current_time: [str(x) for x in range(10)],),
+    ],
+)
+def test_dynamic_partitions_keys(partition_fn: Callable[[Optional[datetime]], List[str]]):
+    partitions = DynamicPartitionsDefinition(partition_fn)
+
+    assert [(p.name, p.value) for p in partitions.get_partitions()] == [
+        (p, p) for p in partition_fn(None)
+    ]
+
+    assert partitions.get_partition_keys() == partition_fn(None)
+
+
+def test_partitions_def_to_string():
+    hourly = HourlyPartitionsDefinition(
+        "Tue Jan 11 1:30PM", "America/Los_Angeles", "%a %b %d %I:%M%p"
+    )
+    assert str(hourly) == "Hourly, starting Thu Jan 11 01:30PM America/Los_Angeles."
+
+    daily = DailyPartitionsDefinition(start_date="2020-01-01", end_offset=1)
+    assert str(daily) == "Daily, starting 2020-01-01 UTC. End offsetted by 1 partition."
+
+    static = StaticPartitionsDefinition(["foo", "bar", "baz", "qux"])
+    assert str(static) == "'foo', 'bar', 'baz', 'qux'"
+
+    dynamic_fn = lambda _current_time: ["a_partition"]
+    dynamic = DynamicPartitionsDefinition(dynamic_fn)
+    assert str(dynamic) == "'a_partition'"
