@@ -1,4 +1,3 @@
-import os
 import tempfile
 import time
 from contextlib import contextmanager
@@ -28,7 +27,7 @@ from dagster.core.storage.event_log import (
     InMemoryEventLogStorage,
     SqliteEventLogStorage,
 )
-from dagster.core.storage.event_log.migration import migrate_asset_key_data
+from dagster.core.storage.event_log.migration import ASSET_KEY_INDEX_COLS, migrate_asset_key_data
 from dagster.core.storage.noop_compute_log_manager import NoOpComputeLogManager
 from dagster.core.storage.root import LocalArtifactStorage
 from dagster.core.storage.runs import InMemoryRunStorage
@@ -101,18 +100,6 @@ def solid_normalization(_):
     yield Output(1)
 
 
-@solid
-def solid_asset_tags(_):
-    yield AssetMaterialization(asset_key=AssetKey("asset_tags"), tags={"foo": "FOO", "bar": "BAR"})
-    yield Output(1)
-
-
-@solid
-def solid_asset_tags_overwrite(_):
-    yield AssetMaterialization(asset_key=AssetKey("asset_tags"), tags={"foo": "NOT_FOO"})
-    yield Output(1)
-
-
 @pipeline
 def pipeline_one():
     solid_one()
@@ -129,16 +116,6 @@ def pipeline_normalization():
     solid_normalization()
 
 
-@pipeline
-def pipeline_asset_tags():
-    solid_asset_tags()
-
-
-@pipeline
-def pipeline_asset_tags_overwrite():
-    solid_asset_tags_overwrite()
-
-
 @solid(config_schema={"partition": Field(str, is_required=False)})
 def solid_partitioned(context):
     yield AssetMaterialization(
@@ -502,3 +479,180 @@ def test_backcompat_asset_read():
             _validate_instance_assets(instance)
             my_job.execute_in_process(instance=instance)
             _validate_instance_assets(instance)
+
+
+def test_backcompat_asset_materializations():
+    src_dir = file_relative_path(__file__, "compat_tests/snapshot_0_11_0_asset_materialization")
+    # should contain materialization events for asset keys a, b, c, d, e, f
+    # events a and b have been wiped, but b has been rematerialized
+
+    @op
+    def materialize():
+        yield AssetMaterialization(AssetKey("c"), tags={"foo": "bar"})
+        yield Output(None)
+
+    @job
+    def my_job():
+        materialize()
+
+    def _validate_materialization(asset_key, event, expected_tags):
+        assert isinstance(event, EventLogEntry)
+        assert event.dagster_event
+        assert event.dagster_event.is_step_materialization
+        assert event.dagster_event.step_materialization_data.materialization.asset_key == asset_key
+        assert event.dagster_event.step_materialization_data.materialization.tags == expected_tags
+
+    a = AssetKey("a")
+    b = AssetKey("b")
+    c = AssetKey("c")
+
+    with copy_directory(src_dir) as test_dir:
+        with DagsterInstance.from_ref(InstanceRef.from_dir(test_dir)) as instance:
+            storage = instance.event_log_storage
+
+            a_mat = storage.get_latest_materialization_events([a]).get(a)
+            assert a_mat is None
+
+            b_mat = storage.get_latest_materialization_events([b]).get(b)
+            _validate_materialization(b, b_mat, expected_tags={})
+
+            c_mat = storage.get_latest_materialization_events([c]).get(c)
+            _validate_materialization(c, c_mat, expected_tags={})
+
+            mat_by_key = storage.get_latest_materialization_events([a, b, c])
+            assert mat_by_key.get(a) is None
+            _validate_materialization(b, mat_by_key.get(b), expected_tags={})
+            _validate_materialization(c, mat_by_key.get(c), expected_tags={})
+
+            # materialize c with tags
+            my_job.execute_in_process(instance=instance)
+
+            a_mat = storage.get_latest_materialization_events([a]).get(a)
+            assert a_mat is None
+
+            b_mat = storage.get_latest_materialization_events([b]).get(b)
+            _validate_materialization(b, b_mat, expected_tags={})
+
+            c_mat = storage.get_latest_materialization_events([c]).get(c)
+            _validate_materialization(c, c_mat, expected_tags={"foo": "bar"})
+
+            mat_by_key = storage.get_latest_materialization_events([a, b, c])
+            assert mat_by_key.get(a) is None
+            _validate_materialization(b, mat_by_key.get(b), expected_tags={})
+            _validate_materialization(c, c_mat, expected_tags={"foo": "bar"})
+
+
+def test_asset_lazy_migration():
+    src_dir = file_relative_path(__file__, "compat_tests/snapshot_0_11_0_asset_materialization")
+    # should contain materialization events for asset keys a, b, c, d, e, f
+    # events a and b have been wiped, but b has been rematerialized
+
+    @op
+    def materialize():
+        yield AssetMaterialization(AssetKey("a"))
+        yield AssetMaterialization(AssetKey("b"))
+        yield AssetMaterialization(AssetKey("c"))
+        yield AssetMaterialization(AssetKey("d"))
+        yield AssetMaterialization(AssetKey("e"))
+        yield AssetMaterialization(AssetKey("f"))
+        yield Output(None)
+
+    @job
+    def my_job():
+        materialize()
+
+    with copy_directory(src_dir) as test_dir:
+        with DagsterInstance.from_ref(InstanceRef.from_dir(test_dir)) as instance:
+            storage = instance.event_log_storage
+            assert not storage.has_asset_key_index_cols()
+            assert not storage.has_secondary_index(ASSET_KEY_INDEX_COLS)
+
+            # run the schema migration without reindexing the asset keys
+            storage.upgrade()
+            assert storage.has_asset_key_index_cols()
+            assert not storage.has_secondary_index(ASSET_KEY_INDEX_COLS)
+
+            # fetch all asset keys
+            instance.all_asset_keys()
+            assert not storage.has_secondary_index(ASSET_KEY_INDEX_COLS)
+
+            # wipe a, b in order to populate wipe_timestamp
+            storage.wipe_asset(AssetKey("a"))
+            storage.wipe_asset(AssetKey("b"))
+
+            # materialize all the assets to populate materialization_timestamp
+            my_job.execute_in_process(instance=instance)
+
+            # still should not be migrated (on write)
+            assert not storage.has_secondary_index(ASSET_KEY_INDEX_COLS)
+
+            # fetching partial results should not trigger migration
+            instance.get_asset_keys(prefix=["b"])
+            instance.get_asset_keys(cursor=str(AssetKey("b")))
+            instance.get_latest_materialization_events(asset_keys=[AssetKey("b")])
+
+            assert not storage.has_secondary_index(ASSET_KEY_INDEX_COLS)
+
+            # on read, we should see that all the data has already been migrated and we can now mark
+            # the asset key index as migrated
+            instance.all_asset_keys()
+            assert storage.has_secondary_index(ASSET_KEY_INDEX_COLS)
+
+
+@asset_test
+def test_get_materialization_count_by_partition(asset_aware_context):
+    src_dir = file_relative_path(__file__, "compat_tests/snapshot_0_11_0_asset_materialization")
+
+    d = AssetKey("c")
+
+    with copy_directory(src_dir) as test_dir:
+        with DagsterInstance.from_ref(InstanceRef.from_dir(test_dir)) as instance:
+            storage = instance.event_log_storage
+
+            materialization_count_by_key = storage.get_materialization_count_by_partition([d])
+
+            assert materialization_count_by_key.get(d) == {}
+
+    a = AssetKey("no_materializations_asset")
+    b = AssetKey("no_partitions_asset")
+    c = AssetKey("two_partitions_asset")
+
+    @op
+    def materialize():
+        yield AssetMaterialization(b)
+        yield AssetMaterialization(c, partition="a")
+        yield Output(None)
+
+    @job
+    def my_job():
+        materialize()
+
+    @op
+    def materialize_two():
+        yield AssetMaterialization(c, partition="a")
+        yield AssetMaterialization(c, partition="b")
+        yield Output(None)
+
+    @job
+    def job_two():
+        materialize_two()
+
+    with asset_aware_context() as ctx:
+        instance, event_log_storage = ctx
+        my_job.execute_in_process(instance=instance)
+
+        materialization_count_by_key = event_log_storage.get_materialization_count_by_partition(
+            [a, b, c]
+        )
+
+        assert materialization_count_by_key.get(a) == {}
+        assert materialization_count_by_key.get(b) == {}
+        assert materialization_count_by_key.get(c)["a"] == 1
+        assert len(materialization_count_by_key.get(c)) == 1
+
+        job_two.execute_in_process(instance=instance)
+        materialization_count_by_key = event_log_storage.get_materialization_count_by_partition(
+            [a, b, c]
+        )
+        assert materialization_count_by_key.get(c)["a"] == 2
+        assert materialization_count_by_key.get(c)["b"] == 1
