@@ -15,6 +15,7 @@ from dagster import (
     OutputDefinition,
     RetryRequested,
     pipeline,
+    resource,
     seven,
     solid,
 )
@@ -164,11 +165,17 @@ def _event_record(run_id, solid_name, timestamp, event_type, event_specific_data
 
 
 def _mode_def(event_callback):
+    @resource
+    def foo_resource():
+        time.sleep(0.1)
+        return "foo"
+
     return ModeDefinition(
+        resource_defs={"foo": foo_resource},
         logger_defs={
             "callback": construct_event_logger(event_callback),
             "console": colored_console_logger,
-        }
+        },
     )
 
 
@@ -237,7 +244,10 @@ class TestEventLogStorage:
     @pytest.fixture(name="storage", params=[])
     def event_log_storage(self, request):
         with request.param() as s:
-            yield s
+            try:
+                yield s
+            finally:
+                s.dispose()
 
     def test_init_log_storage(self, storage):
         if isinstance(storage, InMemoryEventLogStorage):
@@ -447,16 +457,19 @@ class TestEventLogStorage:
         assert a_stats.step_key == "A"
         assert a_stats.status.value == "SUCCESS"
         assert a_stats.end_time - a_stats.start_time == 100
+        assert len(a_stats.attempts_list) == 1
 
         b_stats = [stats for stats in step_stats if stats.step_key == "B"][0]
         assert b_stats.step_key == "B"
         assert b_stats.status.value == "FAILURE"
         assert b_stats.end_time - b_stats.start_time == 50
+        assert len(b_stats.attempts_list) == 1
 
         c_stats = [stats for stats in step_stats if stats.step_key == "C"][0]
         assert c_stats.step_key == "C"
         assert c_stats.status.value == "SKIPPED"
         assert c_stats.end_time - c_stats.start_time == 25
+        assert len(c_stats.attempts_list) == 1
 
         d_stats = [stats for stats in step_stats if stats.step_key == "D"][0]
         assert d_stats.step_key == "D"
@@ -464,6 +477,7 @@ class TestEventLogStorage:
         assert d_stats.end_time - d_stats.start_time == 150
         assert len(d_stats.materializations) == 3
         assert len(d_stats.expectation_results) == 2
+        assert len(c_stats.attempts_list) == 1
 
     def test_secondary_index(self, storage):
         if not isinstance(storage, SqlEventLogStorage):
@@ -1013,10 +1027,12 @@ class TestEventLogStorage:
         assert step_stats[0].status == StepEventStatus.SUCCESS
         assert step_stats[0].end_time > step_stats[0].start_time
         assert step_stats[0].attempts == 1
+        assert len(step_stats[0].attempts_list) == 1
         assert step_stats[1].step_key == "should_fail"
         assert step_stats[1].status == StepEventStatus.FAILURE
         assert step_stats[1].end_time > step_stats[0].start_time
         assert step_stats[1].attempts == 1
+        assert len(step_stats[1].attempts_list) == 1
 
     def test_run_step_stats_with_retries(self, storage):
         @solid(input_defs=[InputDefinition("_input", str)], output_defs=[OutputDefinition(str)])
@@ -1036,6 +1052,75 @@ class TestEventLogStorage:
         assert step_stats[0].status == StepEventStatus.FAILURE
         assert step_stats[0].end_time > step_stats[0].start_time
         assert step_stats[0].attempts == 4
+        assert len(step_stats[0].attempts_list) == 4
+
+    # After adding the IN_PROGRESS field to the StepEventStatus enum, tests in internal fail
+    # Temporarily skipping this test
+    @pytest.mark.skip
+    def test_run_step_stats_with_in_progress(self, storage):
+        def _in_progress_run_records(run_id):
+            now = time.time()
+            return [
+                _event_record(run_id, "A", now - 325, DagsterEventType.STEP_START),
+                _event_record(run_id, "C", now - 175, DagsterEventType.STEP_START),
+                _event_record(run_id, "C", now - 150, DagsterEventType.STEP_SKIPPED),
+                _event_record(run_id, "D", now - 150, DagsterEventType.STEP_START),
+                _event_record(run_id, "D", now - 150, DagsterEventType.STEP_UP_FOR_RETRY),
+                _event_record(run_id, "E", now - 150, DagsterEventType.STEP_START),
+                _event_record(run_id, "E", now - 150, DagsterEventType.STEP_UP_FOR_RETRY),
+                _event_record(run_id, "E", now - 125, DagsterEventType.STEP_RESTARTED),
+            ]
+
+        for record in _in_progress_run_records(run_id=DEFAULT_RUN_ID):
+            storage.store_event(record)
+
+        step_stats = storage.get_step_stats_for_run(DEFAULT_RUN_ID)
+
+        assert len(step_stats) == 4
+
+        assert step_stats[0].step_key == "A"
+        assert step_stats[0].status == StepEventStatus.IN_PROGRESS
+        assert not step_stats[0].end_time
+        assert step_stats[0].attempts == 1
+        assert len(step_stats[0].attempts_list) == 1
+
+        assert step_stats[1].step_key == "C"
+        assert step_stats[1].status == StepEventStatus.SKIPPED
+        assert step_stats[1].end_time > step_stats[1].start_time
+        assert step_stats[1].attempts == 1
+        assert len(step_stats[1].attempts_list) == 1
+
+        assert step_stats[2].step_key == "D"
+        assert step_stats[2].status == StepEventStatus.IN_PROGRESS
+        assert not step_stats[2].end_time
+        assert step_stats[2].attempts == 1
+        assert len(step_stats[2].attempts_list) == 1
+
+        assert step_stats[3].step_key == "E"
+        assert step_stats[3].status == StepEventStatus.IN_PROGRESS
+        assert not step_stats[3].end_time
+        assert step_stats[3].attempts == 2
+        assert len(step_stats[3].attempts_list) == 2
+
+    def test_run_step_stats_with_resource_markers(self, storage):
+        @solid(required_resource_keys={"foo"})
+        def foo_solid():
+            pass
+
+        def _pipeline():
+            foo_solid()
+
+        events, result = _synthesize_events(_pipeline, check_success=False)
+        for event in events:
+            storage.store_event(event)
+
+        step_stats = storage.get_step_stats_for_run(result.run_id)
+        assert len(step_stats) == 1
+        assert step_stats[0].step_key == "foo_solid"
+        assert step_stats[0].status == StepEventStatus.SUCCESS
+        assert step_stats[0].end_time > step_stats[0].start_time
+        assert len(step_stats[0].markers) == 1
+        assert step_stats[0].markers[0].end_time >= step_stats[0].markers[0].start_time + 0.1
 
     def test_get_event_records(self, storage):
         if isinstance(storage, SqliteEventLogStorage):
@@ -1248,6 +1333,8 @@ class TestEventLogStorage:
         assert len(event_list) == len(safe_events)
         assert all([isinstance(event, EventLogEntry) for event in event_list])
 
+    # https://github.com/dagster-io/dagster/issues/5127
+    @pytest.mark.skip
     def test_watch_unwatch(self, storage):
         if not self.can_watch():
             pytest.skip("storage cannot watch runs")
@@ -1291,3 +1378,145 @@ class TestEventLogStorage:
 
         assert len(event_list) == len(safe_events)
         assert all([isinstance(event, EventLogEntry) for event in event_list])
+
+    def test_engine_event_markers(self, storage):
+        @solid
+        def return_one(_):
+            return 1
+
+        @pipeline
+        def a_pipe():
+            return_one()
+
+        with instance_for_test() as instance:
+            if not storage._instance:  # pylint: disable=protected-access
+                storage.register_instance(instance)
+
+            run_id = make_new_run_id()
+            run = instance.create_run_for_pipeline(a_pipe, run_id=run_id)
+
+            instance.report_engine_event(
+                "blah blah", run, EngineEventData(marker_start="FOO"), step_key="return_one"
+            )
+            instance.report_engine_event(
+                "blah blah", run, EngineEventData(marker_end="FOO"), step_key="return_one"
+            )
+            logs = storage.get_logs_for_run(run_id)
+            for entry in logs:
+                assert entry.step_key == "return_one"
+
+    def test_latest_materializations(self, storage):
+        @solid
+        def one(_):
+            yield AssetMaterialization(AssetKey("a"), tags={"num": str(1)})
+            yield AssetMaterialization(AssetKey("b"), tags={"num": str(1)})
+            yield AssetMaterialization(AssetKey("c"), tags={"num": str(1)})
+            yield AssetMaterialization(AssetKey("d"), tags={"num": str(1)})
+            yield Output(1)
+
+        @solid
+        def two(_):
+            yield AssetMaterialization(AssetKey("b"), tags={"num": str(2)})
+            yield AssetMaterialization(AssetKey("c"), tags={"num": str(2)})
+            yield Output(2)
+
+        def _event_tags(event):
+            return event.dagster_event.step_materialization_data.materialization.tags
+
+        def _fetch_events(storage):
+            return storage.get_latest_materialization_events(
+                [
+                    AssetKey("a"),
+                    AssetKey("b"),
+                    AssetKey("c"),
+                    AssetKey("d"),
+                ]
+            )
+
+        events, _ = _synthesize_events(lambda: one())
+        for event in events:
+            storage.store_event(event)
+
+        events_by_key = _fetch_events(storage)
+        assert len(events_by_key) == 4
+        assert _event_tags(events_by_key[AssetKey("a")])["num"] == "1"
+        assert _event_tags(events_by_key[AssetKey("b")])["num"] == "1"
+        assert _event_tags(events_by_key[AssetKey("c")])["num"] == "1"
+        assert _event_tags(events_by_key[AssetKey("d")])["num"] == "1"
+
+        # wipe 2 of the assets, make sure we respect that
+        if self.can_wipe():
+            storage.wipe_asset(AssetKey("a"))
+            storage.wipe_asset(AssetKey("b"))
+            events_by_key = _fetch_events(storage)
+            assert events_by_key.get(AssetKey("a")) is None
+            assert events_by_key.get(AssetKey("b")) is None
+            assert _event_tags(events_by_key[AssetKey("c")])["num"] == "1"
+            assert _event_tags(events_by_key[AssetKey("d")])["num"] == "1"
+
+            # rematerialize one of the wiped assets, one of the existing assets
+            events, _ = _synthesize_events(lambda: two())
+            for event in events:
+                storage.store_event(event)
+
+            events_by_key = _fetch_events(storage)
+            assert events_by_key.get(AssetKey("a")) is None
+            assert _event_tags(events_by_key[AssetKey("b")])["num"] == "2"
+            assert _event_tags(events_by_key[AssetKey("c")])["num"] == "2"
+            assert _event_tags(events_by_key[AssetKey("d")])["num"] == "1"
+
+        else:
+            events, _ = _synthesize_events(lambda: two())
+            for event in events:
+                storage.store_event(event)
+            events_by_key = _fetch_events(storage)
+            assert _event_tags(events_by_key[AssetKey("a")])["num"] == "1"
+            assert _event_tags(events_by_key[AssetKey("b")])["num"] == "2"
+            assert _event_tags(events_by_key[AssetKey("c")])["num"] == "2"
+            assert _event_tags(events_by_key[AssetKey("d")])["num"] == "1"
+
+    def test_get_materialization_count_by_partition(self, storage):
+        a = AssetKey("a")
+        b = AssetKey("b")
+
+        @solid
+        def one(_):
+            yield AssetMaterialization(a, partition="a")
+            yield AssetMaterialization(b, partition="b")
+            yield AssetMaterialization(b, partition="b")
+            yield AssetMaterialization(b, partition="c")
+            yield Output(1)
+
+        @solid
+        def two(_):
+            yield AssetMaterialization(a, partition="d")
+            yield Output(2)
+
+        def _fetch_counts(storage):
+            return storage.get_materialization_count_by_partition([a, b])
+
+        events, _ = _synthesize_events(lambda: one())
+        for event in events:
+            storage.store_event(event)
+
+        materialization_count_by_partition = _fetch_counts(storage)
+        assert materialization_count_by_partition[a]["a"] == 1
+        assert materialization_count_by_partition[b]["b"] == 2
+        assert materialization_count_by_partition[b]["c"] == 1
+
+        # wipe asset, make sure we respect that
+        if self.can_wipe():
+            storage.wipe_asset(a)
+            materialization_count_by_partition = _fetch_counts(storage)
+            assert materialization_count_by_partition.get(a) == {}
+            assert materialization_count_by_partition[b]["b"] == 2
+
+            # rematerialize wiped asset
+            events, _ = _synthesize_events(lambda: two())
+            for event in events:
+                storage.store_event(event)
+
+            materialization_count_by_partition = _fetch_counts(storage)
+            assert materialization_count_by_partition[a]["d"] == 1
+            assert len(materialization_count_by_partition[a]) == 1
+            assert materialization_count_by_partition[b]["b"] == 2
