@@ -14,6 +14,7 @@ from dagster import (
     check,
     execute_pipeline,
     file_relative_path,
+    job,
     pipeline,
     solid,
 )
@@ -573,3 +574,35 @@ def test_pipeline_run_status_dagster_run_status():
     result = _deserialize_json(test_str, legacy_env)
     assert isinstance(result, PipelineRunStatus)
     assert result.value == test_status.value
+
+
+def test_start_time_end_time():
+    src_dir = file_relative_path(__file__, "snapshot_0_13_12_pre_add_start_time_and_end_time")
+    with copy_directory(src_dir) as test_dir:
+
+        @job
+        def _test():
+            pass
+
+        db_path = os.path.join(test_dir, "history", "runs.db")
+        assert get_current_alembic_version(db_path) == "7f2b1a4ca7a5"
+        assert "start_time" not in set(get_sqlite3_columns(db_path, "runs"))
+        assert "end_time" not in set(get_sqlite3_columns(db_path, "runs"))
+
+        # this migration was optional, so make sure things work before migrating
+        instance = DagsterInstance.from_ref(InstanceRef.from_dir(test_dir))
+        assert "start_time" not in set(get_sqlite3_columns(db_path, "runs"))
+        assert "end_time" not in set(get_sqlite3_columns(db_path, "runs"))
+        assert instance.get_run_records()
+        assert instance.create_run_for_pipeline(_test)
+
+        instance.upgrade()
+
+        # Make sure the schema is migrated
+        assert "start_time" in set(get_sqlite3_columns(db_path, "runs"))
+        assert instance.get_run_records()
+        assert instance.create_run_for_pipeline(_test)
+
+        instance._run_storage._alembic_downgrade(rev="7f2b1a4ca7a5")
+
+        assert get_current_alembic_version(db_path) == "7f2b1a4ca7a5"
